@@ -54,6 +54,7 @@
 #include <errno.h>
 #include <time.h>
 #include <ctype.h>
+#include <limits.h>
 
 #include "drmtest.h"
 #include "intel_chipset.h"
@@ -156,10 +157,10 @@
  *   test logic.
  *
  * - When adding a new feature test function which uses igt_skip() internally,
- *   use the &lt;prefix&gt;_require_&lt;feature_name&gt; naming scheme. When you
+ *   use the <prefix>_require_<feature_name> naming scheme. When you
  *   instead add a feature test function which returns a boolean, because your
  *   main test logic must take different actions depending upon the feature's
- *   availability, then instead use the &lt;prefix&gt;_has_&lt;feature_name&gt;.
+ *   availability, then instead use the <prefix>_has_<feature_name>.
  *
  * - As already mentioned eschew explicit error handling logic as much as
  *   possible. If your test absolutely has to handle the error of some function
@@ -248,6 +249,11 @@ static struct {
 } log_buffer;
 static pthread_mutex_t log_buffer_mutex = PTHREAD_MUTEX_INITIALIZER;
 
+const char *igt_test_name(void)
+{
+	return command_str;
+}
+
 static void _igt_log_buffer_append(char *line)
 {
 	pthread_mutex_lock(&log_buffer_mutex);
@@ -362,7 +368,7 @@ void __igt_fixture_end(void)
 	assert(in_fixture);
 
 	in_fixture = false;
-	longjmp(igt_subtest_jmpbuf, 1);
+	siglongjmp(igt_subtest_jmpbuf, 1);
 }
 
 /*
@@ -390,6 +396,7 @@ static void low_mem_killer_disable(bool disable)
 	/* The following must persist across invocations */
 	static char prev_adj_scores[256];
 	static int adj_scores_len = 0;
+	static bool is_disabled = false;
 
 	/* capture the permissions bits for the lowmemkiller adj pseudo-file.
 	 * Bail out if the stat fails; it probably means that there is no
@@ -402,7 +409,7 @@ static void low_mem_killer_disable(bool disable)
 	/* make sure the file can be read/written - by default it is write-only */
 	chmod(adj_fname, S_IRUSR | S_IWUSR);
 
-	if (disable) {
+	if (disable && !is_disabled) {
 		/* read the current oom adj parameters for lowmemorykiller */
 		fd = open(adj_fname, O_RDWR);
 		igt_assert(fd != -1);
@@ -412,14 +419,18 @@ static void low_mem_killer_disable(bool disable)
 		/* writing 9999 to this module parameter effectively diables the
 		 * low memory killer. This is not a real file, so we dont need to
 		 * seek to the start or truncate it */
-		write(fd, no_lowmem_killer, sizeof(no_lowmem_killer));
+		igt_assert_eq(write(fd, no_lowmem_killer, sizeof(no_lowmem_killer)),
+			      sizeof(no_lowmem_killer));
 		close(fd);
-	} else {
+		is_disabled = true;
+	} else if (is_disabled) {
 		/* just re-enstate the original settings */
 		fd = open(adj_fname, O_WRONLY);
 		igt_assert(fd != -1);
-		write(fd, prev_adj_scores, adj_scores_len);
+		igt_assert_eq(write(fd, prev_adj_scores, adj_scores_len),
+			      adj_scores_len);
 		close(fd);
+		is_disabled = false;
 	}
 
 	/* re-enstate the file permissions */
@@ -429,7 +440,9 @@ static void low_mem_killer_disable(bool disable)
 bool igt_exit_called;
 static void common_exit_handler(int sig)
 {
-	low_mem_killer_disable(false);
+	if (!igt_only_list_subtests()) {
+		low_mem_killer_disable(false);
+	}
 
 	/* When not killed by a signal check that igt_exit() has been properly
 	 * called. */
@@ -482,14 +495,14 @@ static void oom_adjust_for_doom(void)
 	igt_assert(write(fd, always_kill, sizeof(always_kill)) == sizeof(always_kill));
 	close(fd);
 
-	low_mem_killer_disable(true);
 }
 
 static int common_init(int *argc, char **argv,
 		       const char *extra_short_opts,
 		       struct option *extra_long_opts,
 		       const char *help_str,
-		       igt_opt_handler_t extra_opt_handler)
+		       igt_opt_handler_t extra_opt_handler,
+		       void *handler_data)
 {
 	int c, option_index = 0, i, x;
 	static struct option long_options[] = {
@@ -615,7 +628,7 @@ static int common_init(int *argc, char **argv,
 			ret = -2;
 			goto out;
 		default:
-			ret = extra_opt_handler(c, option_index);
+			ret = extra_opt_handler(c, option_index, handler_data);
 			if (ret)
 				goto out;
 		}
@@ -645,6 +658,7 @@ out:
 		print_version();
 
 		oom_adjust_for_doom();
+		low_mem_killer_disable(true);
 	}
 
 	/* install exit handler, to ensure we clean up */
@@ -670,6 +684,7 @@ out:
  * @extra_long_opts: getopt_long() compliant list with additional long options
  * @help_str: help string for the additional options
  * @extra_opt_handler: handler for the additional options
+ * @handler_data: user data given to @extra_opt_handler when invoked
  *
  * This function handles the subtest related cmdline options and allows an
  * arbitrary set of additional options. This is useful for tests which have
@@ -685,13 +700,14 @@ int igt_subtest_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
 				struct option *extra_long_opts,
 				const char *help_str,
-				igt_opt_handler_t extra_opt_handler)
+				igt_opt_handler_t extra_opt_handler,
+				void *handler_data)
 {
 	int ret;
 
 	test_with_subtests = true;
 	ret = common_init(argc, argv, extra_short_opts, extra_long_opts,
-			  help_str, extra_opt_handler);
+			  help_str, extra_opt_handler, handler_data);
 
 	return ret;
 }
@@ -706,6 +722,7 @@ enum igt_log_level igt_log_level = IGT_LOG_INFO;
  * @extra_long_opts: getopt_long() compliant list with additional long options
  * @help_str: help string for the additional options
  * @extra_opt_handler: handler for the additional options
+ * @handler_data: user data given to @extra_opt_handler when invoked
  *
  * This initializes a simple test without any support for subtests and allows
  * an arbitrary set of additional options.
@@ -714,10 +731,11 @@ void igt_simple_init_parse_opts(int *argc, char **argv,
 				const char *extra_short_opts,
 				struct option *extra_long_opts,
 				const char *help_str,
-				igt_opt_handler_t extra_opt_handler)
+				igt_opt_handler_t extra_opt_handler,
+				void *handler_data)
 {
 	common_init(argc, argv, extra_short_opts, extra_long_opts, help_str,
-		    extra_opt_handler);
+		    extra_opt_handler, handler_data);
 }
 
 /*
@@ -808,7 +826,7 @@ static void exit_subtest(const char *result)
 
 	printf("Subtest %s: %s (%.3fs)\n", in_subtest, result, elapsed);
 	in_subtest = NULL;
-	longjmp(igt_subtest_jmpbuf, 1);
+	siglongjmp(igt_subtest_jmpbuf, 1);
 }
 
 /**
@@ -859,7 +877,8 @@ void __igt_skip_check(const char *file, const int line,
 	char *err_str = NULL;
 
 	if (err)
-		asprintf(&err_str, "Last errno: %i, %s\n", err, strerror(err));
+		igt_assert_neq(asprintf(&err_str, "Last errno: %i, %s\n", err, strerror(err)),
+			       -1);
 
 	if (f) {
 		static char *buf;
@@ -869,7 +888,7 @@ void __igt_skip_check(const char *file, const int line,
 			free(buf);
 
 		va_start(args, f);
-		vasprintf(&buf, f, args);
+		igt_assert_neq(vasprintf(&buf, f, args), -1);
 		va_end(args);
 
 		igt_skip("Test requirement not met in function %s, file %s:%i:\n"
@@ -918,6 +937,8 @@ void igt_fail(int exitcode)
 {
 	assert(exitcode != IGT_EXIT_SUCCESS && exitcode != IGT_EXIT_SKIP);
 
+	igt_debug_wait_for_keypress("failure");
+
 	if (!failed_one)
 		igt_exitcode = exitcode;
 
@@ -955,8 +976,35 @@ static bool run_under_gdb(void)
 		strncmp(basename(buf), "gdb", 3) == 0);
 }
 
-void __igt_fail_assert(int exitcode, const char *domain, const char *file,
-		       const int line, const char *func, const char *assertion,
+#ifdef HAVE_LIBUNWIND
+#define UNW_LOCAL_ONLY
+#include <libunwind.h>
+
+static void print_backtrace(void)
+{
+	unw_cursor_t cursor;
+	unw_context_t uc;
+	int stack_num = 0;
+
+	printf("Stack trace:\n");
+
+	unw_getcontext(&uc);
+	unw_init_local(&cursor, &uc);
+	while (unw_step(&cursor) > 0) {
+		char name[255];
+		unw_word_t off;
+
+		if (unw_get_proc_name(&cursor, name, 255, &off) < 0)
+			strcpy(name, "<unknown>");
+
+		printf("  #%d [%s+0x%x]\n", stack_num++, name,
+		       (unsigned int) off);
+	}
+}
+#endif
+
+void __igt_fail_assert(const char *domain, const char *file, const int line,
+		       const char *func, const char *assertion,
 		       const char *f, ...)
 {
 	va_list args;
@@ -976,9 +1024,13 @@ void __igt_fail_assert(int exitcode, const char *domain, const char *file,
 		va_end(args);
 	}
 
+#ifdef HAVE_LIBUNWIND
+	print_backtrace();
+#endif
+
 	if (run_under_gdb())
 		abort();
-	igt_fail(exitcode);
+	igt_fail(IGT_EXIT_FAILURE);
 }
 
 /**
@@ -1302,9 +1354,12 @@ static igt_exit_handler_t exit_handler_fn[MAX_EXIT_HANDLERS];
 static bool exit_handler_disabled;
 static sigset_t saved_sig_mask;
 #define SIGDEF(x) { x, #x, sizeof(#x) - 1 }
+#define SILENT(x) { x, NULL, 0 }
 static const struct { int number; const char *name; size_t name_len; } handled_signals[] =
-	{ SIGDEF(SIGINT), SIGDEF(SIGHUP), SIGDEF(SIGTERM), SIGDEF(SIGQUIT),
-	  SIGDEF(SIGPIPE), SIGDEF(SIGABRT), SIGDEF(SIGSEGV), SIGDEF(SIGBUS) };
+	{ SILENT(SIGINT), SILENT(SIGHUP), SILENT(SIGTERM), SILENT(SIGQUIT),
+	  SILENT(SIGPIPE), SIGDEF(SIGABRT), SIGDEF(SIGSEGV), SIGDEF(SIGBUS) };
+#undef SILENT
+#undef SIGDEF
 
 static int install_sig_handler(int sig_num, sighandler_t handler)
 {
@@ -1355,22 +1410,48 @@ static void igt_atexit_handler(void)
 		call_exit_handlers(0);
 }
 
+static bool crash_signal(int sig)
+{
+	switch (sig) {
+	case SIGILL:
+	case SIGBUS:
+	case SIGSEGV:
+		return true;
+	default:
+		return false;
+	}
+}
+
 static void fatal_sig_handler(int sig)
 {
 	int i;
 
-	restore_all_sig_handler();
-
 	for (i = 0; i < ARRAY_SIZE(handled_signals); i++) {
-		if (handled_signals[i].number == sig) {
-			write(STDERR_FILENO, "Received signal ", 16);
-			write(STDERR_FILENO, handled_signals[i].name,
-			      handled_signals[i].name_len);
-			write(STDERR_FILENO, ".\n", 2);
-			break;
+		if (handled_signals[i].number != sig)
+			continue;
+
+		if (handled_signals[i].name_len) {
+			igt_assert_eq(write(STDERR_FILENO, "Received signal ", 16),
+                                      16);
+			igt_assert_eq(write(STDERR_FILENO, handled_signals[i].name, handled_signals[i].name_len),
+                                      handled_signals[i].name_len);
+			igt_assert_eq(write(STDERR_FILENO, ".\n", 2), 2);
 		}
+
+		if (in_subtest && crash_signal(sig)) {
+			/* Linux standard to return exit code as 128 + signal */
+			if (!failed_one)
+				igt_exitcode = 128 + sig;
+
+			failed_one = true;
+
+			exit_subtest("CRASH");
+		}
+		break;
 	}
 
+	restore_all_sig_handler();
+
 	/*
 	 * exit_handler_disabled is always false here, since when we set it
 	 * we also block signals.
@@ -1655,15 +1736,17 @@ out:
 
 static void igt_alarm_handler(int signal)
 {
-	/* exit with timeout status */
-	igt_fail(IGT_EXIT_TIMEOUT);
+	igt_info("Timed out\n");
+
+	/* exit with failure status */
+	igt_fail(IGT_EXIT_FAILURE);
 }
 
 /**
  * igt_set_timeout:
  * @seconds: number of seconds before timeout
  *
- * Fail a test and exit with #IGT_EXIT_TIMEOUT status after the specified
+ * Fail a test and exit with #IGT_EXIT_FAILURE status after the specified
  * number of seconds have elapsed. If the current test has subtests and the
  * timeout occurs outside a subtest, subsequent subtests will be skipped and
  * marked as failed.
@@ -1686,3 +1769,23 @@ void igt_set_timeout(unsigned int seconds)
 
 	alarm(seconds);
 }
+
+FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
+		       const char* filename)
+{
+	char path[PATH_MAX];
+	FILE *fp;
+
+	snprintf(path, sizeof(path), "%s/%s", igt_datadir, filename);
+	fp = fopen(path, "r");
+	if (!fp) {
+		snprintf(path, sizeof(path), "%s/%s", igt_srcdir, filename);
+		fp = fopen(path, "r");
+	}
+
+	if (!fp)
+		igt_critical("Could not open data file \"%s\": %s", filename,
+			     strerror(errno));
+
+	return fp;
+}
