@@ -35,6 +35,7 @@
 #include <i915_drm.h>
 
 #include "drmtest.h"
+#include "igt_aux.h"
 #include "igt_kms.h"
 #include "igt_debugfs.h"
 
@@ -51,7 +52,7 @@
  * # Pipe CRC Support
  *
  * This library wraps up the kernel's support for capturing pipe CRCs into a
- * neat and tidy package. For the datailed usage see all the functions which
+ * neat and tidy package. For the detailed usage see all the functions which
  * work on #igt_pipe_crc_t. This is supported on all platforms and outputs.
  *
  * Actually using pipe CRCs to write modeset tests is a bit tricky though, so
@@ -59,11 +60,11 @@
  * blending, color correction and other hardware and how exactly the CRC is
  * computed at each tap point vary by hardware generation and are not disclosed.
  *
- * The only way to use #igt_crc_t CRCs therefors is to compare CRCs among each
+ * The only way to use #igt_crc_t CRCs therefore is to compare CRCs among each
  * another either for equality or difference. Otherwise CRCs must be treated as
  * completely opaque values. Note that not even CRCs from different pipes or tap
- * points on the same platform can be compared. Hence only use igt_crc_is_null()
- * and igt_crc_equal() to inspect CRC values captured by the same
+ * points on the same platform can be compared. Hence only use
+ * igt_assert_crc_equal() to inspect CRC values captured by the same
  * #igt_pipe_crc_t object.
  *
  * # Other debugfs interface wrappers
@@ -142,7 +143,7 @@ static igt_debugfs_t *__igt_debugfs_singleton(void)
  * @mode: mode bits as used by open()
  *
  * This opens a debugfs file as a Unix file descriptor. The filename should be
- * relative to the drm device's root, i.e without "drm/&lt;minor&gt;".
+ * relative to the drm device's root, i.e without "drm/<minor>".
  *
  * Returns:
  * The Unix file descriptor for the debugfs file or -1 if that didn't work out.
@@ -165,7 +166,7 @@ int igt_debugfs_open(const char *filename, int mode)
  * @mode: mode string as used by fopen()
  *
  * This opens a debugfs file as a libc FILE. The filename should be
- * relative to the drm device's root, i.e without "drm/&lt;minor&gt;".
+ * relative to the drm device's root, i.e without "drm/<minor>".
  *
  * Returns:
  * The libc FILE pointer for the debugfs file or NULL if that didn't work out.
@@ -189,49 +190,22 @@ FILE *igt_debugfs_fopen(const char *filename,
  */
 
 /**
- * igt_crc_is_null:
- * @crc: pipe CRC value to check
- *
- * Returns: True if the CRC is null/invalid, false if it represents a captured
- * valid CRC.
- */
-bool igt_crc_is_null(igt_crc_t *crc)
-{
-	int i;
-
-	for (i = 0; i < crc->n_words; i++) {
-		igt_warn_on_f(crc->crc[i] == 0xffffffff,
-			      "Suspicious CRC: it looks like the CRC "
-			      "read back was from a register in a powered "
-			      "down well\n");
-		if (crc->crc[i])
-			return false;
-	}
-
-	return true;
-}
-
-/**
- * igt_crc_equal:
+ * igt_assert_crc_equal:
  * @a: first pipe CRC value
  * @b: second pipe CRC value
  *
- * Compares two CRC values.
- *
- * Returns: Retruns true if the two CRCs match, false otherwise.
+ * Compares two CRC values and fails the testcase if they don't match with
+ * igt_fail(). Note that due to CRC collisions CRC based testcase can only
+ * assert that CRCs match, never that they are different. Otherwise there might
+ * be random testcase failures when different screen contents end up with the
+ * same CRC by chance.
  */
-bool igt_crc_equal(igt_crc_t *a, igt_crc_t *b)
+void igt_assert_crc_equal(igt_crc_t *a, igt_crc_t *b)
 {
 	int i;
 
-	if (a->n_words != b->n_words)
-		return false;
-
 	for (i = 0; i < a->n_words; i++)
-		if (a->crc[i] != b->crc[i])
-			return false;
-
-	return true;
+		igt_assert_eq_u32(a->crc[i], b->crc[i]);
 }
 
 /**
@@ -294,10 +268,13 @@ static bool igt_pipe_crc_do_start(igt_pipe_crc_t *pipe_crc)
 {
 	char buf[64];
 
+	/* Stop first just to make sure we don't have lingering state left. */
+	igt_pipe_crc_stop(pipe_crc);
+
 	sprintf(buf, "pipe %s %s", kmstest_pipe_name(pipe_crc->pipe),
 		pipe_crc_source_name(pipe_crc->source));
 	errno = 0;
-	write(pipe_crc->ctl_fd, buf, strlen(buf));
+	igt_assert_eq(write(pipe_crc->ctl_fd, buf, strlen(buf)), strlen(buf));
 	if (errno != 0)
 		return false;
 
@@ -309,7 +286,7 @@ static void igt_pipe_crc_pipe_off(int fd, enum pipe pipe)
 	char buf[32];
 
 	sprintf(buf, "pipe %s none", kmstest_pipe_name(pipe));
-	write(fd, buf, strlen(buf));
+	igt_assert_eq(write(fd, buf, strlen(buf)), strlen(buf));
 }
 
 static void igt_pipe_crc_reset(void)
@@ -362,9 +339,9 @@ void igt_require_pipe_crc(void)
  *
  * This sets up a new pipe CRC capture object for the given @pipe and @source.
  *
- * Returns: A pipe CRC object if the given @pipe and @source is available, NULL
- * otherwise. Tests can use this to intelligently skip if they require a
- * specific pipe CRC source to function properly.
+ * Returns: A pipe CRC object if the given @pipe and @source. The library
+ * assumes that the source is always available since recent kernels support at
+ * least INTEL_PIPE_CRC_SOURCE_AUTO everywhere.
  */
 igt_pipe_crc_t *
 igt_pipe_crc_new(enum pipe pipe, enum intel_pipe_crc_source source)
@@ -388,14 +365,6 @@ igt_pipe_crc_new(enum pipe pipe, enum intel_pipe_crc_source source)
 	pipe_crc->pipe = pipe;
 	pipe_crc->source = source;
 
-	/* make sure this source is actually supported */
-	if (!igt_pipe_crc_do_start(pipe_crc)) {
-		igt_pipe_crc_free(pipe_crc);
-		return NULL;
-	}
-
-	igt_pipe_crc_stop(pipe_crc);
-
 	return pipe_crc;
 }
 
@@ -430,8 +399,11 @@ void igt_pipe_crc_start(igt_pipe_crc_t *pipe_crc)
 	/*
 	 * For some no yet identified reason, the first CRC is bonkers. So
 	 * let's just wait for the next vblank and read out the buggy result.
+	 *
+	 * On CHV sometimes the second CRC is bonkers as well, so don't trust
+	 * that one either.
 	 */
-	igt_pipe_crc_get_crcs(pipe_crc, 1, &crcs);
+	igt_pipe_crc_get_crcs(pipe_crc, 2, &crcs);
 	free(crcs);
 }
 
@@ -446,7 +418,7 @@ void igt_pipe_crc_stop(igt_pipe_crc_t *pipe_crc)
 	char buf[32];
 
 	sprintf(buf, "pipe %s none", kmstest_pipe_name(pipe_crc->pipe));
-	write(pipe_crc->ctl_fd, buf, strlen(buf));
+	igt_assert_eq(write(pipe_crc->ctl_fd, buf, strlen(buf)), strlen(buf));
 }
 
 static bool pipe_crc_init_from_string(igt_crc_t *crc, const char *line)
@@ -511,6 +483,23 @@ igt_pipe_crc_get_crcs(igt_pipe_crc_t *pipe_crc, int n_crcs,
 	*out_crcs = crcs;
 }
 
+static void crc_sanity_checks(igt_crc_t *crc)
+{
+	int i;
+	bool all_zero = true;
+
+	for (i = 0; i < crc->n_words; i++) {
+		igt_warn_on_f(crc->crc[i] == 0xffffffff,
+			      "Suspicious CRC: it looks like the CRC "
+			      "read back was from a register in a powered "
+			      "down well\n");
+		if (crc->crc[i])
+			all_zero = false;
+	}
+
+	igt_warn_on_f(all_zero, "Suspicious CRC: All values are 0.\n");
+}
+
 /**
  * igt_pipe_crc_collect_crc:
  * @pipe_crc: pipe CRC object
@@ -521,14 +510,19 @@ igt_pipe_crc_get_crcs(igt_pipe_crc_t *pipe_crc, int n_crcs,
  *
  * This function takes care of the pipe_crc book-keeping, it will start/stop
  * the collection of the CRC.
+ *
+ * This function also calls the interactive debug with the "crc" domain, so you
+ * can make use of this feature to actually see the screen that is being CRC'd.
  */
 void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc)
 {
+	igt_debug_wait_for_keypress("crc");
+
 	igt_pipe_crc_start(pipe_crc);
 	read_one_crc(pipe_crc, out_crc);
 	igt_pipe_crc_stop(pipe_crc);
 
-	igt_assert(!igt_crc_is_null(out_crc));
+	crc_sanity_checks(out_crc);
 }
 
 /*
@@ -553,7 +547,9 @@ void igt_drop_caches_set(uint64_t val)
 	fd = igt_debugfs_open("i915_gem_drop_caches", O_WRONLY);
 
 	igt_assert(fd >= 0);
-	nbytes = write(fd, data, strlen(data) + 1);
+	do {
+		nbytes = write(fd, data, strlen(data) + 1);
+	} while (nbytes == -1 && (errno == EINTR || errno == EAGAIN));
 	igt_assert(nbytes == strlen(data) + 1);
 	close(fd);
 }
@@ -613,125 +609,3 @@ void igt_enable_prefault(void)
 {
 	igt_prefault_control(true);
 }
-
-/**
- * igt_open_forcewake_handle:
- *
- * This functions opens the debugfs forcewake file and so prevents the GT from
- * suspending. The reference is automatically dropped when the is closed.
- *
- * Returns:
- * The file descriptor of the forcewake handle or -1 if that didn't work out.
- */
-int igt_open_forcewake_handle(void)
-{
-	if (getenv("IGT_NO_FORCEWAKE"))
-		return -1;
-	return igt_debugfs_open("i915_forcewake_user", O_WRONLY);
-}
-
-/**
- * igt_to_stop_ring_flag:
- * @ring: the specified ring flag from execbuf ioctl (I915_EXEC_*)
- *
- * This converts the specified ring to a ring flag to be used
- * with igt_get_stop_rings() and igt_set_stop_rings().
- *
- * Returns:
- * Ring flag for the given ring.
- */
-enum stop_ring_flags igt_to_stop_ring_flag(int ring) {
-	if (ring == I915_EXEC_DEFAULT)
-		return STOP_RING_RENDER;
-
-	igt_assert(ring && ((ring & ~I915_EXEC_RING_MASK) == 0));
-	return 1 << (ring - 1);
-}
-
-static void stop_rings_write(uint32_t mask)
-{
-	int fd;
-	char buf[80];
-
-	igt_assert(snprintf(buf, sizeof(buf), "0x%08x", mask) == 10);
-	fd = igt_debugfs_open("i915_ring_stop", O_WRONLY);
-	igt_assert(fd >= 0);
-
-	igt_assert(write(fd, buf, strlen(buf)) == strlen(buf));
-	close(fd);
-}
-
-/**
- * igt_get_stop_rings:
- *
- * Read current ring flags from 'i915_ring_stop' debugfs entry.
- *
- * Returns:
- * Current ring flags.
- */
-enum stop_ring_flags igt_get_stop_rings(void)
-{
-	int fd;
-	char buf[80];
-	int l;
-	unsigned long long ring_mask;
-
-	fd = igt_debugfs_open("i915_ring_stop", O_RDONLY);
-	igt_assert(fd >= 0);
-	l = read(fd, buf, sizeof(buf)-1);
-	igt_assert(l > 0);
-	igt_assert(l < sizeof(buf));
-
-	buf[l] = '\0';
-
-	close(fd);
-
-	errno = 0;
-	ring_mask = strtoull(buf, NULL, 0);
-	igt_assert(errno == 0);
-	return ring_mask;
-}
-
-/**
- * igt_set_stop_rings:
- * @flags: Ring flags to write
- *
- * This writes @flags to 'i915_ring_stop' debugfs entry. Driver will
- * prevent the CPU from writing tail pointer for the ring that @flags
- * specify. Note that the ring is not stopped right away. Instead any
- * further command emissions won't be executed after the flag is set.
- *
- * This is the least invasive way to make the GPU stuck. Hence you must
- * set this after a batch submission with it's own invalid or endless
- * looping instructions. In this case it is merely for giving notification
- * for the driver that this was simulated hang, as the batch would have
- * caused hang in any case. On the other hand if you use a valid or noop
- * batch and want to hang the ring (GPU), you must set corresponding flag
- * before submitting the batch.
- *
- * Driver checks periodically if a ring is making any progress, and if
- * it is not, it will declare the ring to be hung and will reset the GPU.
- * After reset, the driver will clear flags in 'i915_ring_stop'
- *
- * Note: Always when hanging the GPU, use igt_set_stop_rings() to
- * notify the driver. Driver controls hang log messaging based on
- * these flags and thus prevents false positives on logs.
- */
-void igt_set_stop_rings(enum stop_ring_flags flags)
-{
-	enum stop_ring_flags current;
-
-	igt_assert((flags & ~(STOP_RING_ALL |
-			      STOP_RING_ALLOW_BAN |
-			      STOP_RING_ALLOW_ERRORS)) == 0);
-
-	current = igt_get_stop_rings();
-	igt_assert_f(flags == 0 || current == 0,
-		     "previous i915_ring_stop is still 0x%x\n", current);
-
-	stop_rings_write(flags);
-	current = igt_get_stop_rings();
-	igt_warn_on_f(current != flags,
-		      "i915_ring_stop readback mismatch 0x%x vs 0x%x\n",
-		      flags, current);
-}
