@@ -62,7 +62,7 @@ static struct format_desc_struct {
 	int depth;
 } format_desc[] = {
 	DF(RGB565,	RGB16_565,	16, 16),
-	DF(RGB888,	INVALID,	24, 24),
+	//DF(RGB888,	INVALID,	24, 24),
 	DF(XRGB8888,	RGB24,		32, 24),
 	DF(XRGB2101010,	RGB30,		32, 30),
 	DF(ARGB8888,	ARGB32,		32, 32),
@@ -75,15 +75,16 @@ static struct format_desc_struct {
 
 /* helpers to create nice-looking framebuffers */
 static int create_bo_for_fb(int fd, int width, int height, int bpp,
-			    bool tiled, uint32_t *gem_handle_ret,
-			    unsigned *size_ret, unsigned *stride_ret,
-			    unsigned bo_size)
+			    uint64_t tiling, unsigned bo_size,
+			    uint32_t *gem_handle_ret,
+			    unsigned *size_ret,
+			    unsigned *stride_ret)
 {
 	uint32_t gem_handle;
 	int size, ret = 0;
 	unsigned stride;
 
-	if (tiled) {
+	if (tiling != LOCAL_DRM_FORMAT_MOD_NONE) {
 		int v;
 
 		/* Round the tiling up to the next power-of-two and the
@@ -111,7 +112,7 @@ static int create_bo_for_fb(int fd, int width, int height, int bpp,
 		bo_size = size;
 	gem_handle = gem_create(fd, bo_size);
 
-	if (tiled)
+	if (tiling == LOCAL_I915_FORMAT_MOD_X_TILED)
 		ret = __gem_set_tiling(fd, gem_handle, I915_TILING_X, stride);
 
 	*stride_ret = stride;
@@ -337,6 +338,15 @@ void igt_paint_test_pattern(cairo_t *cr, int width, int height)
 	igt_assert(!cairo_status(cr));
 }
 
+static cairo_status_t
+stdio_read_func(void *closure, unsigned char* data, unsigned int size)
+{
+	if (fread(data, 1, size, (FILE*)closure) != size)
+		return CAIRO_STATUS_READ_ERROR;
+
+	return CAIRO_STATUS_SUCCESS;
+}
+
 /**
  * igt_paint_image:
  * @cr: cairo drawing context
@@ -346,9 +356,8 @@ void igt_paint_test_pattern(cairo_t *cr, int width, int height)
  * @dst_width: width of the destination rectangle
  * @dst_height: height of the destination rectangle
  *
- * This function can be used to draw a scaled version of the supplied png image.
- * This is currently only used by the CR-code based testing in the "testdisplay"
- * testcase.
+ * This function can be used to draw a scaled version of the supplied png image,
+ * which is loaded from the package data directory.
  */
 void igt_paint_image(cairo_t *cr, const char *filename,
 		     int dst_x, int dst_y, int dst_width, int dst_height)
@@ -356,8 +365,11 @@ void igt_paint_image(cairo_t *cr, const char *filename,
 	cairo_surface_t *image;
 	int img_width, img_height;
 	double scale_x, scale_y;
+	FILE* f;
 
-	image = cairo_image_surface_create_from_png(filename);
+	f = igt_fopen_data(filename);
+
+	image = cairo_image_surface_create_from_png_stream(&stdio_read_func, f);
 	igt_assert(cairo_surface_status(image) == CAIRO_STATUS_SUCCESS);
 
 	img_width = cairo_image_surface_get_width(image);
@@ -376,6 +388,8 @@ void igt_paint_image(cairo_t *cr, const char *filename,
 	cairo_surface_destroy(image);
 
 	cairo_restore(cr);
+
+	fclose(f);
 }
 
 /**
@@ -384,7 +398,7 @@ void igt_paint_image(cairo_t *cr, const char *filename,
  * @width: width of the framebuffer in pixel
  * @height: height of the framebuffer in pixel
  * @format: drm fourcc pixel format code
- * @tiled: X-tiled or linear framebuffer
+ * @tiling: tiling layout of the framebuffer (as framebuffer modifier)
  * @fb: pointer to an #igt_fb structure
  * @bo_size: size of the backing bo (0 for minimum needed size)
  *
@@ -396,43 +410,53 @@ void igt_paint_image(cairo_t *cr, const char *filename,
  * for rgb pixel formats.
  *
  * Returns:
- * The kms id of the created framebuffer on success or a negative error code on
- * failure.
+ * The kms id of the created framebuffer.
  */
-unsigned int igt_create_fb_with_bo_size(int fd, int width, int height,
-					uint32_t format, bool tiled,
-					struct igt_fb *fb, unsigned bo_size)
+unsigned int
+igt_create_fb_with_bo_size(int fd, int width, int height,
+			   uint32_t format, uint64_t tiling,
+			   struct igt_fb *fb, unsigned bo_size)
 {
-	uint32_t handles[4];
-	uint32_t pitches[4];
-	uint32_t offsets[4];
 	uint32_t fb_id;
 	int bpp;
-	int ret;
 
 	memset(fb, 0, sizeof(*fb));
 
 	bpp = igt_drm_format_to_bpp(format);
-	ret = create_bo_for_fb(fd, width, height, bpp, tiled, &fb->gem_handle,
-			      &fb->size, &fb->stride, bo_size);
-	if (ret < 0)
-		return ret;
 
-	memset(handles, 0, sizeof(handles));
-	handles[0] = fb->gem_handle;
-	memset(pitches, 0, sizeof(pitches));
-	pitches[0] = fb->stride;
-	memset(offsets, 0, sizeof(offsets));
-	if (drmModeAddFB2(fd, width, height, format, handles, pitches,
-			  offsets, &fb_id, 0) < 0) {
-		gem_close(fd, fb->gem_handle);
+	igt_debug("%s(width=%d, height=%d, format=0x%x [bpp=%d], tiling=0x%"PRIx64", size=%d\n",
+		  __func__, width, height, format, bpp, tiling, bo_size);
+	do_or_die(create_bo_for_fb(fd, width, height, bpp, tiling, bo_size,
+				   &fb->gem_handle, &fb->size, &fb->stride));
 
-		return 0;
+	igt_debug("%s(handle=%d, pitch=%d)\n",
+		  __func__, fb->gem_handle, fb->stride);
+
+	if (tiling != LOCAL_DRM_FORMAT_MOD_NONE &&
+	    tiling != LOCAL_I915_FORMAT_MOD_X_TILED) {
+		do_or_die(__kms_addfb(fd, fb->gem_handle, width, height,
+				      fb->stride, format, tiling,
+				      LOCAL_DRM_MODE_FB_MODIFIERS, &fb_id));
+	} else {
+		uint32_t handles[4];
+		uint32_t pitches[4];
+		uint32_t offsets[4];
+
+		memset(handles, 0, sizeof(handles));
+		memset(pitches, 0, sizeof(pitches));
+		memset(offsets, 0, sizeof(offsets));
+
+		handles[0] = fb->gem_handle;
+		pitches[0] = fb->stride;
+
+		do_or_die(drmModeAddFB2(fd, width, height, format,
+					handles, pitches, offsets,
+					&fb_id, 0));
 	}
 
 	fb->width = width;
 	fb->height = height;
-	fb->tiling = tiled;
+	fb->tiling = tiling;
 	fb->drm_format = format;
 	fb->fb_id = fb_id;
 
@@ -445,7 +469,7 @@ unsigned int igt_create_fb_with_bo_size(int fd, int width, int height,
  * @width: width of the framebuffer in pixel
  * @height: height of the framebuffer in pixel
  * @format: drm fourcc pixel format code
- * @tiled: X-tiled or linear framebuffer
+ * @tiling: tiling layout of the framebuffer
  * @fb: pointer to an #igt_fb structure
  *
  * This function allocates a gem buffer object suitable to back a framebuffer
@@ -456,13 +480,12 @@ unsigned int igt_create_fb_with_bo_size(int fd, int width, int height,
  * for rgb pixel formats.
  *
  * Returns:
- * The kms id of the created framebuffer on success or a negative error code on
- * failure.
+ * The kms id of the created framebuffer.
  */
 unsigned int igt_create_fb(int fd, int width, int height, uint32_t format,
-			   bool tiled, struct igt_fb *fb)
+			   uint64_t tiling, struct igt_fb *fb)
 {
-	return igt_create_fb_with_bo_size(fd, width, height, format, tiled, fb, 0);
+	return igt_create_fb_with_bo_size(fd, width, height, format, tiling, fb, 0);
 }
 
 /**
@@ -471,7 +494,7 @@ unsigned int igt_create_fb(int fd, int width, int height, uint32_t format,
  * @width: width of the framebuffer in pixel
  * @height: height of the framebuffer in pixel
  * @format: drm fourcc pixel format code
- * @tiled: X-tiled or linear framebuffer
+ * @tiling: tiling layout of the framebuffer
  * @r: red value to use as fill color
  * @g: gree value to use as fill color
  * @b: blue value to use as fill color
@@ -489,14 +512,14 @@ unsigned int igt_create_fb(int fd, int width, int height, uint32_t format,
  * failure.
  */
 unsigned int igt_create_color_fb(int fd, int width, int height,
-				 uint32_t format, bool tiled,
+				 uint32_t format, uint64_t tiling,
 				 double r, double g, double b,
 				 struct igt_fb *fb /* out */)
 {
 	unsigned int fb_id;
 	cairo_t *cr;
 
-	fb_id = igt_create_fb(fd, width, height, format, tiled, fb);
+	fb_id = igt_create_fb(fd, width, height, format, tiling, fb);
 	igt_assert(fb_id);
 
 	cr = igt_get_cairo_ctx(fd, fb);
@@ -507,6 +530,106 @@ unsigned int igt_create_color_fb(int fd, int width, int height,
 	return fb_id;
 }
 
+struct box {
+	int x, y, width, height;
+};
+
+struct stereo_fb_layout {
+	int fb_width, fb_height;
+	struct box left, right;
+};
+
+static void box_init(struct box *box, int x, int y, int bwidth, int bheight)
+{
+	box->x = x;
+	box->y = y;
+	box->width = bwidth;
+	box->height = bheight;
+}
+
+
+static void stereo_fb_layout_from_mode(struct stereo_fb_layout *layout,
+				       drmModeModeInfo *mode)
+{
+	unsigned int format = mode->flags & DRM_MODE_FLAG_3D_MASK;
+	const int hdisplay = mode->hdisplay, vdisplay = mode->vdisplay;
+	int middle;
+
+	switch (format) {
+	case DRM_MODE_FLAG_3D_TOP_AND_BOTTOM:
+		layout->fb_width = hdisplay;
+		layout->fb_height = vdisplay;
+
+		middle = vdisplay / 2;
+		box_init(&layout->left, 0, 0, hdisplay, middle);
+		box_init(&layout->right,
+			 0, middle, hdisplay, vdisplay - middle);
+		break;
+	case DRM_MODE_FLAG_3D_SIDE_BY_SIDE_HALF:
+		layout->fb_width = hdisplay;
+		layout->fb_height = vdisplay;
+
+		middle = hdisplay / 2;
+		box_init(&layout->left, 0, 0, middle, vdisplay);
+		box_init(&layout->right,
+			 middle, 0, hdisplay - middle, vdisplay);
+		break;
+	case DRM_MODE_FLAG_3D_FRAME_PACKING:
+	{
+		int vactive_space = mode->vtotal - vdisplay;
+
+		layout->fb_width = hdisplay;
+		layout->fb_height = 2 * vdisplay + vactive_space;
+
+		box_init(&layout->left,
+			 0, 0, hdisplay, vdisplay);
+		box_init(&layout->right,
+			 0, vdisplay + vactive_space, hdisplay, vdisplay);
+		break;
+	}
+	default:
+		igt_assert(0);
+	}
+}
+
+/**
+ * igt_create_stereo_fb:
+ * @drm_fd: open i915 drm file descriptor
+ * @mode: A stereo 3D mode.
+ * @format: drm fourcc pixel format code
+ * @tiling: tiling layout of the framebuffer
+ *
+ * Create a framebuffer for use with the stereo 3D mode specified by @mode.
+ *
+ * Returns:
+ * The kms id of the created framebuffer on success or a negative error code on
+ * failure.
+ */
+unsigned int igt_create_stereo_fb(int drm_fd, drmModeModeInfo *mode,
+				  uint32_t format, uint64_t tiling)
+{
+	struct stereo_fb_layout layout;
+	cairo_t *cr;
+	uint32_t fb_id;
+	struct igt_fb fb;
+
+	stereo_fb_layout_from_mode(&layout, mode);
+	fb_id = igt_create_fb(drm_fd, layout.fb_width, layout.fb_height, format,
+			      tiling, &fb);
+	cr = igt_get_cairo_ctx(drm_fd, &fb);
+
+	igt_paint_image(cr, "1080p-left.png",
+			layout.left.x, layout.left.y,
+			layout.left.width, layout.left.height);
+	igt_paint_image(cr, "1080p-right.png",
+			layout.right.x, layout.right.y,
+			layout.right.width, layout.right.height);
+
+	cairo_destroy(cr);
+
+	return fb_id;
+}
+
 static cairo_format_t drm_format_to_cairo(uint32_t drm_format)
 {
 	struct format_desc_struct *f;
@@ -515,33 +638,167 @@ static cairo_format_t drm_format_to_cairo(uint32_t drm_format)
 		if (f->drm_id == drm_format)
 			return f->cairo_id;
 
-	igt_fail(101);
+	igt_assert_f(0, "can't find a cairo format for %08x (%s)\n",
+		     drm_format, igt_format_str(drm_format));
 }
 
-static void __destroy_cairo_surface(void *arg)
+struct fb_blit_upload {
+	int fd;
+	struct igt_fb *fb;
+	struct {
+		uint32_t handle;
+		unsigned size, stride;
+		uint8_t *map;
+	} linear;
+};
+
+static unsigned int fb_mod_to_obj_tiling(uint64_t fb_mod)
+{
+	switch (fb_mod) {
+	case LOCAL_DRM_FORMAT_MOD_NONE:
+		return I915_TILING_NONE;
+	case LOCAL_I915_FORMAT_MOD_X_TILED:
+		return I915_TILING_X;
+	case LOCAL_I915_FORMAT_MOD_Y_TILED:
+		return I915_TILING_Y;
+	case LOCAL_I915_FORMAT_MOD_Yf_TILED:
+		return I915_TILING_Yf;
+	default:
+		igt_assert(0);
+	}
+}
+
+static void destroy_cairo_surface__blit(void *arg)
+{
+	struct fb_blit_upload *blit = arg;
+	struct igt_fb *fb = blit->fb;
+	unsigned int obj_tiling = fb_mod_to_obj_tiling(fb->tiling);
+
+	munmap(blit->linear.map, blit->linear.size);
+	fb->cairo_surface = NULL;
+
+	gem_set_domain(blit->fd, blit->linear.handle,
+			I915_GEM_DOMAIN_GTT, 0);
+
+	igt_blitter_fast_copy__raw(blit->fd,
+				   blit->linear.handle,
+				   blit->linear.stride,
+				   I915_TILING_NONE,
+				   0, 0, /* src_x, src_y */
+				   fb->width, fb->height,
+				   fb->gem_handle,
+				   fb->stride,
+				   obj_tiling,
+				   0, 0 /* dst_x, dst_y */);
+
+	gem_sync(blit->fd, blit->linear.handle);
+	gem_close(blit->fd, blit->linear.handle);
+
+	free(blit);
+}
+
+static void create_cairo_surface__blit(int fd, struct igt_fb *fb)
+{
+	struct fb_blit_upload *blit;
+	cairo_format_t cairo_format;
+	unsigned int obj_tiling = fb_mod_to_obj_tiling(fb->tiling);
+	int bpp, ret;
+
+	blit = malloc(sizeof(*blit));
+	igt_assert(blit);
+
+	/*
+	 * We create a linear BO that we'll map for the CPU to write to (using
+	 * cairo). This linear bo will be then blitted to its final
+	 * destination, tiling it at the same time.
+	 */
+	bpp = igt_drm_format_to_bpp(fb->drm_format);
+	ret = create_bo_for_fb(fd, fb->width, fb->height, bpp,
+				LOCAL_DRM_FORMAT_MOD_NONE, 0,
+				&blit->linear.handle,
+				&blit->linear.size,
+				&blit->linear.stride);
+
+	igt_assert(ret == 0);
+
+	blit->fd = fd;
+	blit->fb = fb;
+
+	/* Copy fb content to linear BO */
+	gem_set_domain(fd, blit->linear.handle,
+			I915_GEM_DOMAIN_GTT, 0);
+
+	igt_blitter_fast_copy__raw(fd,
+				   fb->gem_handle,
+				   fb->stride,
+				   obj_tiling,
+				   0, 0, /* src_x, src_y */
+				   fb->width, fb->height,
+				   blit->linear.handle,
+				   blit->linear.stride,
+				   I915_TILING_NONE,
+				   0, 0 /* dst_x, dst_y */);
+
+	gem_sync(fd, blit->linear.handle);
+
+	gem_set_domain(fd, blit->linear.handle,
+		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
+
+	/* Setup cairo context */
+	blit->linear.map = gem_mmap__cpu(fd,
+					 blit->linear.handle,
+					 0,
+					 blit->linear.size,
+					 PROT_READ | PROT_WRITE);
+	igt_assert(blit->linear.map);
+
+	cairo_format = drm_format_to_cairo(fb->drm_format);
+	fb->cairo_surface =
+		cairo_image_surface_create_for_data(blit->linear.map,
+						    cairo_format,
+						    fb->width, fb->height,
+						    blit->linear.stride);
+
+	cairo_surface_set_user_data(fb->cairo_surface,
+				    (cairo_user_data_key_t *)create_cairo_surface__blit,
+				    blit, destroy_cairo_surface__blit);
+}
+
+static void destroy_cairo_surface__gtt(void *arg)
 {
 	struct igt_fb *fb = arg;
+
 	munmap(cairo_image_surface_get_data(fb->cairo_surface), fb->size);
+	fb->cairo_surface = NULL;
+}
+
+static void create_cairo_surface__gtt(int fd, struct igt_fb *fb)
+{
+	fb->cairo_surface =
+		cairo_image_surface_create_for_data(gem_mmap(fd, fb->gem_handle, fb->size, PROT_READ | PROT_WRITE),
+						    drm_format_to_cairo(fb->drm_format),
+						    fb->width, fb->height, fb->stride);
+
+	cairo_surface_set_user_data(fb->cairo_surface,
+				    (cairo_user_data_key_t *)create_cairo_surface__gtt,
+				    fb, destroy_cairo_surface__gtt);
 }
 
 static cairo_surface_t *get_cairo_surface(int fd, struct igt_fb *fb)
 {
 	if (fb->cairo_surface == NULL) {
-		fb->cairo_surface =
-			cairo_image_surface_create_for_data(gem_mmap(fd, fb->gem_handle, fb->size, PROT_READ | PROT_WRITE),
-							    drm_format_to_cairo(fb->drm_format),
-							    fb->width, fb->height, fb->stride);
-
-		cairo_surface_set_user_data(fb->cairo_surface,
-					    (cairo_user_data_key_t *)get_cairo_surface,
-					    fb, __destroy_cairo_surface);
+		if (fb->tiling == LOCAL_I915_FORMAT_MOD_Y_TILED ||
+		    fb->tiling == LOCAL_I915_FORMAT_MOD_Yf_TILED)
+			create_cairo_surface__blit(fd, fb);
+		else
+			create_cairo_surface__gtt(fd, fb);
 	}
 
 	gem_set_domain(fd, fb->gem_handle,
 		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
 
 	igt_assert(cairo_surface_status(fb->cairo_surface) == CAIRO_STATUS_SUCCESS);
-	return cairo_surface_reference(fb->cairo_surface);
+	return fb->cairo_surface;
 }
 
 /**
@@ -628,7 +885,9 @@ uint32_t igt_bpp_depth_to_drm_format(int bpp, int depth)
 		if (f->bpp == bpp && f->depth == depth)
 			return f->drm_id;
 
-	igt_fail(101);
+
+	igt_assert_f(0, "can't find drm format with bpp=%d, depth=%d\n", bpp,
+		     depth);
 }
 
 /**
@@ -647,7 +906,8 @@ uint32_t igt_drm_format_to_bpp(uint32_t drm_format)
 		if (f->drm_id == drm_format)
 			return f->bpp;
 
-	igt_fail(101);
+	igt_assert_f(0, "can't find a bpp format for %08x (%s)\n",
+		     drm_format, igt_format_str(drm_format));
 }
 
 /**
