@@ -306,7 +306,7 @@ void gem_close(int fd, uint32_t handle)
  * This wraps the PWRITE ioctl, which is to upload a linear data to a subrange
  * of a gem buffer object.
  */
-void gem_write(int fd, uint32_t handle, uint32_t offset, const void *buf, uint32_t length)
+void gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf, uint64_t length)
 {
 	struct drm_i915_gem_pwrite gem_pwrite;
 
@@ -329,7 +329,7 @@ void gem_write(int fd, uint32_t handle, uint32_t offset, const void *buf, uint32
  * This wraps the PREAD ioctl, which is to download a linear data to a subrange
  * of a gem buffer object.
  */
-void gem_read(int fd, uint32_t handle, uint32_t offset, void *buf, uint32_t length)
+void gem_read(int fd, uint32_t handle, uint64_t offset, void *buf, uint64_t length)
 {
 	struct drm_i915_gem_pread gem_pread;
 
@@ -371,12 +371,22 @@ void gem_set_domain(int fd, uint32_t handle,
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
  *
- * This is a wrapper around gem_set_domain() which simply blocks for any
- * outstanding rendering to complete.
+ * This functions waits for outstanding rendering to complete.
  */
 void gem_sync(int fd, uint32_t handle)
 {
-	gem_set_domain(fd, handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+	struct drm_i915_gem_wait wait;
+
+	memset(&wait, 0, sizeof(wait));
+	wait.bo_handle = handle;
+	wait.timeout_ns =-1;
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_WAIT, &wait) == 0) {
+		errno = 0;
+		return;
+	}
+
+	gem_set_domain(fd, handle,
+		       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
 }
 
 uint32_t __gem_create(int fd, int size)
@@ -406,7 +416,7 @@ uint32_t __gem_create(int fd, int size)
  *
  * Returns: The file-private handle of the created buffer object
  */
-uint32_t gem_create(int fd, int size)
+uint32_t gem_create(int fd, uint64_t size)
 {
 	struct drm_i915_gem_create create;
 
@@ -429,12 +439,10 @@ uint32_t gem_create(int fd, int size)
  */
 void gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 {
-	int ret;
+	int result;
 
-	ret = drmIoctl(fd,
-		       DRM_IOCTL_I915_GEM_EXECBUFFER2,
-		       execbuf);
-	igt_assert(ret == 0);
+	result = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf);
+	igt_assert(result == 0);
 	errno = 0;
 }
 
@@ -450,7 +458,7 @@ void gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
  *
  * Returns: A pointer to the created memory mapping.
  */
-void *gem_mmap__gtt(int fd, uint32_t handle, int size, int prot)
+void *gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
 {
 	struct drm_i915_gem_mmap_gtt mmap_arg;
 	void *ptr;
@@ -463,15 +471,99 @@ void *gem_mmap__gtt(int fd, uint32_t handle, int size, int prot)
 	ptr = mmap64(0, size, prot, MAP_SHARED, fd, mmap_arg.offset);
 	if (ptr == MAP_FAILED)
 		ptr = NULL;
+	else
+		errno = 0;
 
 	return ptr;
 }
 
+struct local_i915_gem_mmap_v2 {
+	uint32_t handle;
+	uint32_t pad;
+	uint64_t offset;
+	uint64_t size;
+	uint64_t addr_ptr;
+	uint64_t flags;
+#define I915_MMAP_WC 0x1
+};
+#define LOCAL_IOCTL_I915_GEM_MMAP_v2 DRM_IOWR(DRM_COMMAND_BASE + DRM_I915_GEM_MMAP, struct local_i915_gem_mmap_v2)
+
+bool gem_mmap__has_wc(int fd)
+{
+	static int has_wc = -1;
+
+	if (has_wc == -1) {
+		struct drm_i915_getparam gp;
+		int val = -1;
+
+		has_wc = 0;
+
+		memset(&gp, 0, sizeof(gp));
+		gp.param = 30; /* MMAP_VERSION */
+		gp.value = &val;
+
+		/* Do we have the new mmap_ioctl? */
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp);
+		if (val >= 1) {
+			struct local_i915_gem_mmap_v2 arg;
+
+			/* Does this device support wc-mmaps ? */
+			memset(&arg, 0, sizeof(arg));
+			arg.handle = gem_create(fd, 4096);
+			arg.offset = 0;
+			arg.size = 4096;
+			arg.flags = I915_MMAP_WC;
+			has_wc = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg) == 0;
+			gem_close(fd, arg.handle);
+		}
+		errno = 0;
+	}
+
+	return has_wc > 0;
+}
+
+/**
+ * gem_mmap__wc:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @offset: offset in the gem buffer of te mmap arena
+ * @size: size of the mmap arena
+ * @prot: memory protection bits as used by mmap()
+ *
+ * This functions wraps up procedure to establish a memory mapping through
+ * direct cpu access, bypassing the gpu and cpu caches completely and also
+ * bypassing the GTT system agent (i.e. there is no automatic tiling of
+ * the mmapping through the fence registers).
+ *
+ * Returns: A pointer to the created memory mapping.
+ */
+void *gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
+{
+	struct local_i915_gem_mmap_v2 arg;
+
+	if (!gem_mmap__has_wc(fd)) {
+		errno = ENOSYS;
+		return NULL;
+	}
+
+	memset(&arg, 0, sizeof(arg));
+	arg.handle = handle;
+	arg.offset = offset;
+	arg.size = size;
+	arg.flags = I915_MMAP_WC;
+	if (drmIoctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg))
+		return NULL;
+
+	errno = 0;
+	return (void *)(uintptr_t)arg.addr_ptr;
+}
+
 /**
  * gem_mmap__cpu:
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
- * @size: size of the gem buffer
+ * @offset: offset in the gem buffer of te mmap arena
+ * @size: size of the mmap arena
  * @prot: memory protection bits as used by mmap()
  *
  * This functions wraps up procedure to establish a memory mapping through
@@ -479,13 +571,13 @@ void *gem_mmap__gtt(int fd, uint32_t handle, int size, int prot)
  *
  * Returns: A pointer to the created memory mapping.
  */
-void *gem_mmap__cpu(int fd, uint32_t handle, int size, int prot)
+void *gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, unsigned prot)
 {
 	struct drm_i915_gem_mmap mmap_arg;
 
 	memset(&mmap_arg, 0, sizeof(mmap_arg));
 	mmap_arg.handle = handle;
-	mmap_arg.offset = 0;
+	mmap_arg.offset = offset;
 	mmap_arg.size = size;
 	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_MMAP, &mmap_arg))
 		return NULL;
@@ -506,7 +598,7 @@ void *gem_mmap__cpu(int fd, uint32_t handle, int size, int prot)
  * object is in need again it must be set back to WILLNEED before first use.
  *
  * Returns: When setting the madvise state to WILLNEED this returns whether the
- * backing storage was still avialable or not.
+ * backing storage was still available or not.
  */
 int gem_madvise(int fd, uint32_t handle, int state)
 {
@@ -546,6 +638,110 @@ uint32_t gem_context_create(int fd)
 	return create.ctx_id;
 }
 
+int __gem_context_destroy(int fd, uint32_t ctx_id)
+{
+	struct drm_i915_gem_context_destroy destroy;
+	int ret;
+
+	memset(&destroy, 0, sizeof(destroy));
+	destroy.ctx_id = ctx_id;
+
+	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy);
+	if (ret)
+		return -errno;
+	return 0;
+}
+
+/**
+ * gem_context_destroy:
+ * @fd: open i915 drm file descriptor
+ * @ctx_id: i915 hw context id
+ *
+ * This is a wraps the CONTEXT_DESTROY ioctl, which is used to free a hardware
+ * context.
+ */
+void gem_context_destroy(int fd, uint32_t ctx_id)
+{
+	struct drm_i915_gem_context_destroy destroy;
+
+	memset(&destroy, 0, sizeof(destroy));
+	destroy.ctx_id = ctx_id;
+
+	do_ioctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy);
+}
+
+/**
+ * gem_context_get_param:
+ * @fd: open i915 drm file descriptor
+ * @p: i915 hw context parameter
+ *
+ * This is a wraps the CONTEXT_GET_PARAM ioctl, which is used to free a hardware
+ * context. Not that similarly to gem_set_caching() this wrapper calls
+ * igt_require() internally to correctly skip on kernels and platforms where hw
+ * context parameter support is not available.
+ */
+void gem_context_get_param(int fd, struct local_i915_gem_context_param *p)
+{
+#define LOCAL_I915_GEM_CONTEXT_GETPARAM       0x34
+#define LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_GETPARAM, struct local_i915_gem_context_param)
+	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, p);
+}
+
+/**
+ * gem_context_set_param:
+ * @fd: open i915 drm file descriptor
+ * @p: i915 hw context parameter
+ *
+ * This is a wraps the CONTEXT_SET_PARAM ioctl, which is used to free a hardware
+ * context. Not that similarly to gem_set_caching() this wrapper calls
+ * igt_require() internally to correctly skip on kernels and platforms where hw
+ * context parameter support is not available.
+ */
+void gem_context_set_param(int fd, struct local_i915_gem_context_param *p)
+{
+#define LOCAL_I915_GEM_CONTEXT_SETPARAM       0x35
+#define LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_SETPARAM, struct local_i915_gem_context_param)
+	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM, p);
+}
+
+/**
+ * gem_context_require_param:
+ * @fd: open i915 drm file descriptor
+ * @param: i915 hw context parameter
+ *
+ * Feature test macro to query whether hw context parameter support for @param
+ * is available. Automatically skips through igt_require() if not.
+ */
+void gem_context_require_param(int fd, uint64_t param)
+{
+	struct local_i915_gem_context_param p;
+
+	p.context = 0;
+	p.param = param;
+	p.value = 0;
+	p.size = 0;
+
+	igt_require(drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0);
+}
+
+void gem_context_require_ban_period(int fd)
+{
+	static int has_ban_period = -1;
+
+	if (has_ban_period < 0) {
+		struct local_i915_gem_context_param p;
+
+		p.context = 0;
+		p.param = LOCAL_CONTEXT_PARAM_BAN_PERIOD;
+		p.value = 0;
+		p.size = 0;
+
+		has_ban_period = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0;
+	}
+
+	igt_require(has_ban_period);
+}
+
 /**
  * gem_sw_finish:
  * @fd: open i915 drm file descriptor
@@ -597,7 +793,7 @@ bool gem_bo_busy(int fd, uint32_t handle)
  * Feature test macro to check whether the kernel internally uses ppgtt to
  * execute batches. The /aliasing/ in the function name is a bit a misnomer,
  * this driver parameter is also true when full ppgtt address spaces are
- * availabel since for batchbuffer construction only ppgtt or global gtt is
+ * available since for batchbuffer construction only ppgtt or global gtt is
  * relevant.
  *
  * Returns: Whether batches are run through ppgtt.
@@ -619,51 +815,57 @@ bool gem_uses_aliasing_ppgtt(int fd)
 }
 
 /**
- * gem_uses_aliasing_ppgtt:
+ * gem_available_fences:
  * @fd: open i915 drm file descriptor
  *
  * Feature test macro to query the kernel for the number of available fences
- * useable in a batchbuffer. Only relevant for pre-gen4.
+ * usable in a batchbuffer. Only relevant for pre-gen4.
  *
  * Returns: The number of available fences.
  */
 int gem_available_fences(int fd)
 {
-	struct drm_i915_getparam gp;
-	int val = 0;
+	static int num_fences = -1;
 
-	memset(&gp, 0, sizeof(gp));
-	gp.param = I915_PARAM_NUM_FENCES_AVAIL;
-	gp.value = &val;
+	if (num_fences < 0) {
+		struct drm_i915_getparam gp;
 
-	if (ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp)))
-		return 0;
+		memset(&gp, 0, sizeof(gp));
+		gp.param = I915_PARAM_NUM_FENCES_AVAIL;
+		gp.value = &num_fences;
 
-	errno = 0;
-	return val;
+		num_fences = 0;
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp));
+		errno = 0;
+	}
+
+	return num_fences;
 }
 
 bool gem_has_llc(int fd)
 {
-	struct drm_i915_getparam gp;
-	int val = 0;
+	static int has_llc = -1;
 
-	memset(&gp, 0, sizeof(gp));
-	gp.param = I915_PARAM_HAS_LLC;
-	gp.value = &val;
+	if (has_llc < 0) {
+		struct drm_i915_getparam gp;
 
-	if (ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp)))
-		return 0;
+		memset(&gp, 0, sizeof(gp));
+		gp.param = I915_PARAM_HAS_LLC;
+		gp.value = &has_llc;
 
-	errno = 0;
-	return val;
+		has_llc = 0;
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp));
+		errno = 0;
+	}
+
+	return has_llc;
 }
 
 /**
  * gem_get_num_rings:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to query the number of avaible rings. This is useful in
+ * Feature test macro to query the number of available rings. This is useful in
  * test loops which need to step through all rings and similar logic.
  *
  * For more explicit tests of ring availability see gem_has_enable_ring() and
@@ -673,24 +875,26 @@ bool gem_has_llc(int fd)
  */
 int gem_get_num_rings(int fd)
 {
-	int num_rings = 1;	/* render ring is always available */
+	static int num_rings = -1;
 
-	if (gem_has_bsd(fd))
-		num_rings++;
-	else
-		goto skip;
+	if (num_rings < 0) {
+		num_rings = 1;	/* render ring is always available */
 
-	if (gem_has_blt(fd))
-		num_rings++;
-	else
-		goto skip;
-
-	if (gem_has_vebox(fd))
-		num_rings++;
-	else
-		goto skip;
+		if (gem_has_bsd(fd))
+			num_rings++;
+		else
+			goto skip;
 
+		if (gem_has_blt(fd))
+			num_rings++;
+		else
+			goto skip;
 
+		if (gem_has_vebox(fd))
+			num_rings++;
+		else
+			goto skip;
+	}
 skip:
 	return num_rings;
 }
@@ -702,7 +906,7 @@ skip:
  *
  * Feature test macro to query whether a specific ring is available.
  *
- * Returns: Whether the ring is avaible or not.
+ * Returns: Whether the ring is available or not.
  */
 bool gem_has_enable_ring(int fd,int param)
 {
@@ -729,11 +933,14 @@ bool gem_has_enable_ring(int fd,int param)
  *
  * Note that recent Bspec calls this the VCS ring for Video Command Submission.
  *
- * Returns: Whether the BSD ring is avaible or not.
+ * Returns: Whether the BSD ring is available or not.
  */
 bool gem_has_bsd(int fd)
 {
-	return gem_has_enable_ring(fd,I915_PARAM_HAS_BSD);
+	static int has_bsd = -1;
+	if (has_bsd < 0)
+		has_bsd = gem_has_enable_ring(fd,I915_PARAM_HAS_BSD);
+	return has_bsd;
 }
 
 /**
@@ -745,11 +952,14 @@ bool gem_has_bsd(int fd)
  *
  * Note that recent Bspec calls this the BCS ring for Blitter Command Submission.
  *
- * Returns: Whether the blitter ring is avaible or not.
+ * Returns: Whether the blitter ring is available or not.
  */
 bool gem_has_blt(int fd)
 {
-	return gem_has_enable_ring(fd,I915_PARAM_HAS_BLT);
+	static int has_blt = -1;
+	if (has_blt < 0)
+		has_blt =  gem_has_enable_ring(fd,I915_PARAM_HAS_BLT);
+	return has_blt;
 }
 
 #define LOCAL_I915_PARAM_HAS_VEBOX 22
@@ -763,19 +973,41 @@ bool gem_has_blt(int fd)
  * Note that recent Bspec calls this the VECS ring for Video Enhancement Command
  * Submission.
  *
- * Returns: Whether the vebox ring is avaible or not.
+ * Returns: Whether the vebox ring is available or not.
  */
 bool gem_has_vebox(int fd)
 {
-	return gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_VEBOX);
+	static int has_vebox = -1;
+	if (has_vebox < 0)
+		has_vebox =  gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_VEBOX);
+	return has_vebox;
 }
 
+#define LOCAL_I915_PARAM_HAS_BSD2 31
+/**
+ * gem_has_bsd2:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query whether the BSD2 ring is available. This is simply
+ * a specific version of gem_has_enable_ring() for the BSD2 ring.
+ *
+ * Note that recent Bspec calls this the VCS ring for Video Command Submission.
+ *
+ * Returns: Whether the BSD ring is avaible or not.
+ */
+bool gem_has_bsd2(int fd)
+{
+	static int has_bsd2 = -1;
+	if (has_bsd2 < 0)
+		has_bsd2 = gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_BSD2);
+	return has_bsd2;
+}
 /**
  * gem_available_aperture_size:
  * @fd: open i915 drm file descriptor
  *
  * Feature test macro to query the kernel for the available gpu aperture size
- * useable in a batchbuffer.
+ * usable in a batchbuffer.
  *
  * Returns: The available gtt address space size.
  */
@@ -800,21 +1032,26 @@ uint64_t gem_available_aperture_size(int fd)
  */
 uint64_t gem_aperture_size(int fd)
 {
-	struct drm_i915_gem_get_aperture aperture;
+	static uint64_t aperture_size = 0;
 
-	memset(&aperture, 0, sizeof(aperture));
-	aperture.aper_size = 256*1024*1024;
-	do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+	if (aperture_size == 0) {
+		struct drm_i915_gem_get_aperture aperture;
 
-	return aperture.aper_size;
+		memset(&aperture, 0, sizeof(aperture));
+		aperture.aper_size = 256*1024*1024;
+		do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+
+		aperture_size =  aperture.aper_size;
+	}
+
+	return aperture_size;
 }
 
 /**
- * gem_aperture_size:
- * @fd: open i915 drm file descriptor
+ * gem_mappable_aperture_size:
  *
  * Feature test macro to query the kernel for the mappable gpu aperture size.
- * This is the area avaialble for GTT memory mappings.
+ * This is the area available for GTT memory mappings.
  *
  * Returns: The mappable gtt address space size.
  */
@@ -870,10 +1107,10 @@ void gem_require_ring(int fd, int ring_id)
 	case I915_EXEC_RENDER:
 		return;
 	case I915_EXEC_BLT:
-		igt_require(HAS_BLT_RING(intel_get_drm_devid(fd)));
+		igt_require(gem_has_blt(fd));
 		return;
 	case I915_EXEC_BSD:
-		igt_require(HAS_BSD_RING(intel_get_drm_devid(fd)));
+		igt_require(gem_has_bsd(fd));
 		return;
 #ifdef I915_EXEC_VEBOX
 	case I915_EXEC_VEBOX:
@@ -959,3 +1196,51 @@ off_t prime_get_size(int dma_buf_fd)
 	return ret;
 }
 
+/**
+ * igt_require_fb_modifiers:
+ * @fd: Open DRM file descriptor.
+ *
+ * Requires presence of DRM_CAP_ADDFB2_MODIFIERS.
+ */
+void igt_require_fb_modifiers(int fd)
+{
+	static bool has_modifiers, cap_modifiers_tested;
+
+	if (!cap_modifiers_tested) {
+		uint64_t cap_modifiers;
+		int ret;
+
+		ret = drmGetCap(fd, LOCAL_DRM_CAP_ADDFB2_MODIFIERS, &cap_modifiers);
+		igt_assert(ret == 0 || errno == EINVAL);
+		has_modifiers = ret == 0 && cap_modifiers == 1;
+		cap_modifiers_tested = true;
+	}
+
+	igt_require(has_modifiers);
+}
+
+int __kms_addfb(int fd, uint32_t handle, uint32_t width, uint32_t height,
+		uint32_t stride, uint32_t pixel_format, uint64_t modifier,
+		uint32_t flags, uint32_t *buf_id)
+{
+	struct local_drm_mode_fb_cmd2 f;
+	int ret;
+
+	igt_require_fb_modifiers(fd);
+
+	memset(&f, 0, sizeof(f));
+
+	f.width  = width;
+	f.height = height;
+	f.pixel_format = pixel_format;
+	f.flags = flags;
+	f.handles[0] = handle;
+	f.pitches[0] = stride;
+	f.modifier[0] = modifier;
+
+	ret = drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f);
+
+	*buf_id = f.fb_id;
+
+	return ret < 0 ? -errno : ret;
+}
