@@ -54,6 +54,8 @@
 #include "intel_io.h"
 #include "igt_aux.h"
 
+IGT_TEST_DESCRIPTION("Check whether we correctly invalidate the cs tlb.");
+
 #define LOCAL_I915_EXEC_VEBOX (4<<0)
 #define BATCH_SIZE (1024*1024)
 
@@ -116,7 +118,7 @@ static void run_on_ring(int fd, unsigned ring_id, const char *ring_name)
 		igt_progress(buf, split, BATCH_SIZE/8 - 1);
 
 		handle_new = gem_create(fd, BATCH_SIZE);
-		batch_ptr = gem_mmap__cpu(fd, handle_new, BATCH_SIZE,
+		batch_ptr = gem_mmap__cpu(fd, handle_new, 0, BATCH_SIZE,
 					  PROT_READ | PROT_WRITE);
 		batch_ptr[split*2] = MI_BATCH_BUFFER_END;
 
@@ -128,7 +130,8 @@ static void run_on_ring(int fd, unsigned ring_id, const char *ring_name)
 			gem_close(fd, handle);
 		}
 
-		igt_assert(exec(fd, handle_new, split, &gtt_offset_new, 0) == 0);
+		igt_assert_eq(exec(fd, handle_new, split, &gtt_offset_new, ring_id),
+			      0);
 
 		if (split > 0) {
 			/* Check that we've managed to collide in the tlb. */
