@@ -45,9 +45,13 @@
 #include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "intel_chipset.h"
+#include "igt_gt.h"
 
 #include "eviction_common.c"
 
+IGT_TEST_DESCRIPTION("Run a couple of big batches to force the eviction"
+		     " code.");
+
 #define HEIGHT 256
 #define WIDTH 1024
 
@@ -131,7 +135,7 @@ copy(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo)
 
 static void clear(int fd, uint32_t handle, int size)
 {
-	void *base = gem_mmap__cpu(fd, handle, size, PROT_READ | PROT_WRITE);
+	void *base = gem_mmap__cpu(fd, handle, 0, size, PROT_READ | PROT_WRITE);
 
 	igt_assert(base != NULL);
 	memset(base, 0, size);
@@ -151,17 +155,22 @@ static void test_forking_evictions(int fd, int size, int count,
 	int trash_count;
 
 	trash_count = intel_get_total_ram_mb() * 11 / 10;
-	igt_require(intel_check_memory(trash_count, size, CHECK_RAM | CHECK_SWAP));
+	intel_require_memory(trash_count, size, CHECK_RAM | CHECK_SWAP);
 
 	forking_evictions(fd, &fault_ops, size, count, trash_count, flags);
 }
 
+static void test_mlocked_evictions(int fd, int size, int count)
+{
+	mlocked_evictions(fd, &fault_ops, size, count);
+}
+
 static void test_swapping_evictions(int fd, int size, int count)
 {
 	int trash_count;
 
 	trash_count = intel_get_total_ram_mb() * 11 / 10;
-	igt_require(intel_check_memory(trash_count, size, CHECK_RAM | CHECK_SWAP));
+	intel_require_memory(trash_count, size, CHECK_RAM | CHECK_SWAP);
 
 	swapping_evictions(fd, &fault_ops, size, count, trash_count);
 }
@@ -182,8 +191,6 @@ igt_main
 	size = count = 0;
 	fd = -1;
 
-	igt_skip_on_simulation();
-
 	igt_fixture {
 		fd = drm_open_any();
 
@@ -203,6 +210,9 @@ igt_main
 		}
 	}
 
+	igt_subtest("mlocked-normal")
+		test_mlocked_evictions(fd, size, count);
+
 	igt_subtest("swapping-normal")
 		test_swapping_evictions(fd, size, count);
 
@@ -222,6 +232,9 @@ igt_main
 
 	igt_fork_signal_helper();
 
+	igt_subtest("mlocked-interruptible")
+		test_mlocked_evictions(fd, size, count);
+
 	igt_subtest("swapping-interruptible")
 		test_swapping_evictions(fd, size, count);
 
@@ -234,6 +247,29 @@ igt_main
 		test_major_evictions(fd, size, count);
 	}
 
+	if (igt_fork_hang_helper()) {
+		igt_fixture {
+			size = 1024 * 1024;
+			count = 3*gem_aperture_size(fd) / size / 4;
+		}
+
+		igt_subtest("mlocked-hang")
+			test_mlocked_evictions(fd, size, count);
+
+		igt_subtest("swapping-hang")
+			test_swapping_evictions(fd, size, count);
+
+		igt_subtest("minor-hang")
+			test_minor_evictions(fd, size, count);
+
+		igt_subtest("major-hang") {
+			size = 3*gem_aperture_size(fd) / 4;
+			count = 4;
+			test_major_evictions(fd, size, count);
+		}
+
+		igt_stop_hang_helper();
+	}
 	igt_stop_signal_helper();
 
 	igt_fixture {
