@@ -40,10 +40,16 @@
 #include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "igt_debugfs.h"
+#include "igt_aux.h"
+
+#ifndef PAGE_SIZE
+#define PAGE_SIZE 4096
+#endif
 
 static int OBJECT_SIZE = 16*1024*1024;
 
-static void set_domain(int fd, uint32_t handle)
+static void
+set_domain_gtt(int fd, uint32_t handle)
 {
 	gem_set_domain(fd, handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
 }
@@ -123,24 +129,25 @@ test_short(int fd)
 	igt_assert(drmIoctl(fd,
 			    DRM_IOCTL_I915_GEM_MMAP_GTT,
 			    &mmap_arg) == 0);
-	for (pages = 1; pages <= OBJECT_SIZE / 4096; pages <<= 1) {
+	for (pages = 1; pages <= OBJECT_SIZE / PAGE_SIZE; pages <<= 1) {
 		uint8_t *r, *w;
 
-		w = mmap64(0, pages * 4096, PROT_READ | PROT_WRITE,
+		w = mmap64(0, pages * PAGE_SIZE, PROT_READ | PROT_WRITE,
 			   MAP_SHARED, fd, mmap_arg.offset);
 		igt_assert(w != MAP_FAILED);
 
-		r = mmap64(0, pages * 4096, PROT_READ,
+		r = mmap64(0, pages * PAGE_SIZE, PROT_READ,
 			   MAP_SHARED, fd, mmap_arg.offset);
 		igt_assert(r != MAP_FAILED);
 
 		for (p = 0; p < pages; p++) {
-			w[4096*p] = r[4096*p];
-			w[4096*p+4095] = r[4096*p+4095];
+			w[p*PAGE_SIZE] = r[p*PAGE_SIZE];
+			w[p*PAGE_SIZE+(PAGE_SIZE-1)] =
+				r[p*PAGE_SIZE+(PAGE_SIZE-1)];
 		}
 
-		munmap(r, pages * 4096);
-		munmap(w, pages * 4096);
+		munmap(r, pages * PAGE_SIZE);
+		munmap(w, pages * PAGE_SIZE);
 	}
 	gem_close(fd, mmap_arg.handle);
 }
@@ -245,7 +252,7 @@ test_write_gtt(int fd)
 
 	/* prefault object into gtt */
 	dst_gtt = mmap_bo(fd, dst);
-	set_domain(fd, dst);
+	set_domain_gtt(fd, dst);
 	memset(dst_gtt, 0, OBJECT_SIZE);
 	munmap(dst_gtt, OBJECT_SIZE);
 
@@ -257,6 +264,159 @@ test_write_gtt(int fd)
 	munmap(src, OBJECT_SIZE);
 }
 
+static void
+test_huge_bo(int fd, int huge, int tiling)
+{
+	uint32_t bo;
+	char *ptr;
+	char *tiled_pattern;
+	char *linear_pattern;
+	uint64_t size, last_offset;
+	int pitch = tiling == I915_TILING_Y ? 128 : 512;
+	int i;
+
+	switch (huge) {
+	case -1:
+		size = gem_mappable_aperture_size() / 2;
+		break;
+	case 0:
+		size = gem_mappable_aperture_size() + PAGE_SIZE;
+		break;
+	default:
+		size = gem_aperture_size(fd) + PAGE_SIZE;
+		break;
+	}
+	intel_require_memory(1, size, CHECK_RAM);
+
+	last_offset = size - PAGE_SIZE;
+
+	/* Create pattern */
+	bo = gem_create(fd, PAGE_SIZE);
+	if (tiling)
+		gem_set_tiling(fd, bo, tiling, pitch);
+	linear_pattern = gem_mmap__gtt(fd, bo, PAGE_SIZE,
+				       PROT_READ | PROT_WRITE);
+	igt_assert(linear_pattern);
+	for (i = 0; i < PAGE_SIZE; i++)
+		linear_pattern[i] = i;
+	tiled_pattern = gem_mmap__cpu(fd, bo, 0, PAGE_SIZE, PROT_READ);
+	igt_assert(tiled_pattern);
+
+	gem_set_domain(fd, bo, I915_GEM_DOMAIN_CPU | I915_GEM_DOMAIN_GTT, 0);
+	gem_close(fd, bo);
+
+	bo = gem_create(fd, size);
+	if (tiling)
+		gem_set_tiling(fd, bo, tiling, pitch);
+
+	/* Initialise first/last page through CPU mmap */
+	ptr = gem_mmap__cpu(fd, bo, 0, size, PROT_READ | PROT_WRITE);
+	memcpy(ptr, tiled_pattern, PAGE_SIZE);
+	memcpy(ptr + last_offset, tiled_pattern, PAGE_SIZE);
+	munmap(ptr, size);
+
+	/* Obtain mapping for the object through GTT. */
+	ptr = gem_mmap__gtt(fd, bo, size, PROT_READ | PROT_WRITE);
+	igt_require_f(ptr, "Huge BO GTT mapping not supported.\n");
+
+	set_domain_gtt(fd, bo);
+
+	/* Access through GTT should still provide the CPU written values. */
+	igt_assert(memcmp(ptr              , linear_pattern, PAGE_SIZE) == 0);
+	igt_assert(memcmp(ptr + last_offset, linear_pattern, PAGE_SIZE) == 0);
+
+	gem_set_tiling(fd, bo, I915_TILING_NONE, 0);
+
+	igt_assert(memcmp(ptr              , tiled_pattern, PAGE_SIZE) == 0);
+	igt_assert(memcmp(ptr + last_offset, tiled_pattern, PAGE_SIZE) == 0);
+
+	munmap(ptr, size);
+
+	gem_close(fd, bo);
+	munmap(tiled_pattern, PAGE_SIZE);
+	munmap(linear_pattern, PAGE_SIZE);
+}
+
+static void
+test_huge_copy(int fd, int huge, int tiling_a, int tiling_b)
+{
+	uint64_t huge_object_size, i;
+	uint32_t bo, *pattern_a, *pattern_b;
+	char *a, *b;
+
+	switch (huge) {
+	case -2:
+		huge_object_size = gem_mappable_aperture_size() / 4;
+		break;
+	case -1:
+		huge_object_size = gem_mappable_aperture_size() / 2;
+		break;
+	case 0:
+		huge_object_size = gem_mappable_aperture_size() + PAGE_SIZE;
+		break;
+	default:
+		huge_object_size = gem_aperture_size(fd) + PAGE_SIZE;
+		break;
+	}
+	intel_require_memory(2, huge_object_size, CHECK_RAM);
+
+	pattern_a = malloc(PAGE_SIZE);
+	for (i = 0; i < PAGE_SIZE/4; i++)
+		pattern_a[i] = i;
+
+	pattern_b = malloc(PAGE_SIZE);
+	for (i = 0; i < PAGE_SIZE/4; i++)
+		pattern_b[i] = ~i;
+
+	bo = gem_create(fd, huge_object_size);
+	if (tiling_a)
+		gem_set_tiling(fd, bo, tiling_a,
+			       tiling_a == I915_TILING_Y ? 128 : 512);
+	a = gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
+	igt_require(a);
+	gem_close(fd, bo);
+
+	for (i = 0; i < huge_object_size / PAGE_SIZE; i++)
+		memcpy(a + PAGE_SIZE*i, pattern_a, PAGE_SIZE);
+
+	bo = gem_create(fd, huge_object_size);
+	if (tiling_b)
+		gem_set_tiling(fd, bo, tiling_b, 
+			       tiling_b == I915_TILING_Y ? 128 : 512);
+	b = gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
+	igt_require(b);
+	gem_close(fd, bo);
+
+	for (i = 0; i < huge_object_size / PAGE_SIZE; i++)
+		memcpy(b + PAGE_SIZE*i, pattern_b, PAGE_SIZE);
+
+	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+		if (i & 1)
+			memcpy(a + i *PAGE_SIZE, b + i*PAGE_SIZE, PAGE_SIZE);
+		else
+			memcpy(b + i *PAGE_SIZE, a + i*PAGE_SIZE, PAGE_SIZE);
+	}
+
+	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+		if (i & 1)
+			igt_assert(memcmp(pattern_b, a + PAGE_SIZE*i, PAGE_SIZE) == 0);
+		else
+			igt_assert(memcmp(pattern_a, a + PAGE_SIZE*i, PAGE_SIZE) == 0);
+	}
+	munmap(a, huge_object_size);
+
+	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+		if (i & 1)
+			igt_assert(memcmp(pattern_b, b + PAGE_SIZE*i, PAGE_SIZE) == 0);
+		else
+			igt_assert(memcmp(pattern_a, b + PAGE_SIZE*i, PAGE_SIZE) == 0);
+	}
+	munmap(b, huge_object_size);
+
+	free(pattern_a);
+	free(pattern_b);
+}
+
 static void
 test_read(int fd)
 {
@@ -286,7 +446,7 @@ test_write_cpu_read_gtt(int fd)
 	dst = gem_mmap(fd, handle, OBJECT_SIZE, PROT_READ);
 	igt_assert(dst != (uint32_t *)MAP_FAILED);
 
-	src = gem_mmap__cpu(fd, handle, OBJECT_SIZE, PROT_WRITE);
+	src = gem_mmap__cpu(fd, handle, 0, OBJECT_SIZE, PROT_WRITE);
 	igt_assert(src != (uint32_t *)MAP_FAILED);
 
 	gem_close(fd, handle);
@@ -396,6 +556,44 @@ igt_main
 	igt_subtest("write-cpu-read-gtt")
 		test_write_cpu_read_gtt(fd);
 
+	igt_subtest("small-bo")
+		test_huge_bo(fd, -1, I915_TILING_NONE);
+	igt_subtest("small-bo-tiledX")
+		test_huge_bo(fd, -1, I915_TILING_X);
+	igt_subtest("small-bo-tiledY")
+		test_huge_bo(fd, -1, I915_TILING_Y);
+
+	igt_subtest("big-bo")
+		test_huge_bo(fd, 0, I915_TILING_NONE);
+	igt_subtest("big-bo-tiledX")
+		test_huge_bo(fd, 0, I915_TILING_X);
+	igt_subtest("big-bo-tiledY")
+		test_huge_bo(fd, 0, I915_TILING_Y);
+
+	igt_subtest("huge-bo")
+		test_huge_bo(fd, 1, I915_TILING_NONE);
+	igt_subtest("huge-bo-tiledX")
+		test_huge_bo(fd, 1, I915_TILING_X);
+	igt_subtest("huge-bo-tiledY")
+		test_huge_bo(fd, 1, I915_TILING_Y);
+
+	igt_subtest("small-copy")
+		test_huge_copy(fd, -2, I915_TILING_NONE, I915_TILING_NONE);
+	igt_subtest("small-copy-XY")
+		test_huge_copy(fd, -2, I915_TILING_X, I915_TILING_Y);
+	igt_subtest("medium-copy")
+		test_huge_copy(fd, -1, I915_TILING_NONE, I915_TILING_NONE);
+	igt_subtest("medium-copy-XY")
+		test_huge_copy(fd, -1, I915_TILING_X, I915_TILING_Y);
+	igt_subtest("big-copy")
+		test_huge_copy(fd, 0, I915_TILING_NONE, I915_TILING_NONE);
+	igt_subtest("big-copy-XY")
+		test_huge_copy(fd, 0, I915_TILING_X, I915_TILING_Y);
+	igt_subtest("huge-copy")
+		test_huge_copy(fd, 1, I915_TILING_NONE, I915_TILING_NONE);
+	igt_subtest("huge-copy-XY")
+		test_huge_copy(fd, 1, I915_TILING_X, I915_TILING_Y);
+
 	igt_fixture
 		close(fd);
 }
