@@ -35,11 +35,12 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "intel_bufmgr.h"
+#include "intel_chipset.h"
 #include "intel_batchbuffer.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 
 static drm_intel_bufmgr *bufmgr;
 static drm_intel_bo *target_bo;
@@ -55,13 +56,13 @@ store_dword_loop(int divider, unsigned flags)
 	uint32_t *buf;
 	drm_intel_bo *cmd_bo;
 
-	printf("running storedw loop with stall every %i batch\n", divider);
+	igt_info("running storedw loop with stall every %i batch\n", divider);
 
 	cmd = MI_STORE_DWORD_IMM;
 	if (!has_ppgtt)
 		cmd |= MI_MEM_VIRTUAL;
 
-	for (i = 0; i < SLOW_QUICK(0x80000, 4); i++) {
+	for (i = 0; i < SLOW_QUICK(0x2000, 4); i++) {
 		int j = 0;
 		int cmd_address_offset;
 		cmd_bo = drm_intel_bo_alloc(bufmgr, "cmd bo", 4096, 4096);
@@ -83,7 +84,7 @@ store_dword_loop(int divider, unsigned flags)
 			cmd_address_offset = j * 4;
 			buf[j++] = target_bo->offset;
 		}
-		assert(j > 0);
+		igt_assert_lt(0, j);
 		buf[j++] = 0x42000000 + val;
 
 		igt_assert(drm_intel_bo_references(cmd_bo, target_bo) == 0);
@@ -125,7 +126,7 @@ cont:
 		val++;
 	}
 
-	printf("completed %d writes successfully\n", i);
+	igt_info("completed %d writes successfully\n", i);
 }
 
 int fd;
@@ -168,6 +169,22 @@ igt_main
 		store_dword_loop(5, SECURE_DISPATCH);
 	}
 
+	igt_subtest("cached-mapping") {
+		gem_set_caching(fd, target_bo->handle, 1);
+		store_dword_loop(1, 0);
+		store_dword_loop(2, 0);
+		store_dword_loop(3, 0);
+		store_dword_loop(5, 0);
+	}
+
+	igt_subtest("uncached-mapping") {
+		gem_set_caching(fd, target_bo->handle, 0);
+		store_dword_loop(1, 0);
+		store_dword_loop(2, 0);
+		store_dword_loop(3, 0);
+		store_dword_loop(5, 0);
+	}
+
 	igt_fixture {
 		drm_intel_bo_unreference(target_bo);
 		drm_intel_bufmgr_destroy(bufmgr);
