@@ -186,8 +186,8 @@ static void create_fb_for_crtc(struct crtc_config *crtc,
 	fb_id = igt_create_fb(drm_fd, crtc->mode.hdisplay,
 				  crtc->mode.vdisplay,
 				  igt_bpp_depth_to_drm_format(bpp, depth),
-				  I915_TILING_NONE, fb_info);
-	igt_assert(fb_id > 0);
+				  LOCAL_DRM_FORMAT_MOD_NONE, fb_info);
+	igt_assert_lt(0, fb_id);
 }
 
 static void get_mode_for_crtc(struct crtc_config *crtc,
@@ -281,7 +281,7 @@ static void setup_crtcs(drmModeRes *resources, struct connector_config *cconf,
 		unsigned long encoder_mask;
 		int j;
 
-		igt_assert(crtc_count < MAX_CRTCS);
+		igt_assert_lt(crtc_count, MAX_CRTCS);
 
 		crtc->crtc_idx = cconf[i].crtc_idx;
 		drm_crtc = drmModeGetCrtc(drm_fd,
@@ -662,7 +662,7 @@ static void run_test(const struct test_config *tconf)
 		test_combinations(tconf, connector_num);
 }
 
-static int opt_handler(int opt, int opt_index)
+static int opt_handler(int opt, int opt_index, void *data)
 {
 	switch (opt) {
 	case 'd':
@@ -699,8 +699,8 @@ int main(int argc, char **argv)
 	int i;
 	int ret;
 
-	ret = igt_subtest_init_parse_opts(argc, argv, "dt:", NULL, help_str,
-					  opt_handler);
+	ret = igt_subtest_init_parse_opts(&argc, argv, "dt:", NULL, help_str,
+					  opt_handler, NULL);
 	if (ret < 0)
 		return ret == -1 ? 0 : ret;
 
@@ -710,7 +710,7 @@ int main(int argc, char **argv)
 		     "only one of -d and -t is accepted\n");
 
 	igt_fixture {
-		drm_fd = drm_open_any();
+		drm_fd = drm_open_any_master();
 		if (!dry_run)
 			kmstest_set_vt_graphics_mode();
 
