@@ -51,6 +51,7 @@
 
 #include <cairo.h>
 #include <errno.h>
+#include <getopt.h>
 #include <math.h>
 #include <stdint.h>
 #include <stdbool.h>
@@ -67,17 +68,24 @@
 #include "drmtest.h"
 #include "testdisplay.h"
 #include "igt_kms.h"
+#include "igt_core.h"
 
 #include <stdlib.h>
 #include <signal.h>
 
+#define SUBTEST_OPTS	 1
+#define HELP_DESCRIPTION 2
+#define Yb_OPT		 3
+#define Yf_OPT		 4
+
 static int tio_fd;
 struct termios saved_tio;
 
 drmModeRes *resources;
 int drm_fd, modes;
 int test_all_modes = 0, test_preferred_mode = 0, force_mode = 0, test_plane,
-    test_stereo_modes, enable_tiling;
+    test_stereo_modes;
+uint64_t tiling = LOCAL_DRM_FORMAT_MOD_NONE;
 int sleep_between_modes = 5;
 int do_dpms = 0; /* This aliases to DPMS_ON */
 uint32_t depth = 24, stride, bpp;
@@ -118,45 +126,36 @@ static void dump_connectors_fd(int drmfd)
 	drmModeRes *mode_resources = drmModeGetResources(drmfd);
 
 	if (!mode_resources) {
-		fprintf(stderr, "drmModeGetResources failed: %s\n",
-			strerror(errno));
+		igt_warn("drmModeGetResources failed: %s\n", strerror(errno));
 		return;
 	}
 
-	printf("Connectors:\n");
-	printf("id\tencoder\tstatus\t\ttype\tsize (mm)\tmodes\n");
+	igt_info("Connectors:\n");
+	igt_info("id\tencoder\tstatus\t\ttype\tsize (mm)\tmodes\n");
 	for (i = 0; i < mode_resources->count_connectors; i++) {
 		drmModeConnector *connector;
 
 		connector = drmModeGetConnector(drmfd, mode_resources->connectors[i]);
 		if (!connector) {
-			fprintf(stderr, "could not get connector %i: %s\n",
-				mode_resources->connectors[i], strerror(errno));
+			igt_warn("could not get connector %i: %s\n", mode_resources->connectors[i], strerror(errno));
 			continue;
 		}
 
-		printf("%d\t%d\t%s\t%s\t%dx%d\t\t%d\n",
-		       connector->connector_id,
-		       connector->encoder_id,
-		       kmstest_connector_status_str(connector->connection),
-		       kmstest_connector_type_str(connector->connector_type),
-		       connector->mmWidth, connector->mmHeight,
-		       connector->count_modes);
+		igt_info("%d\t%d\t%s\t%s\t%dx%d\t\t%d\n", connector->connector_id, connector->encoder_id, kmstest_connector_status_str(connector->connection), kmstest_connector_type_str(connector->connector_type), connector->mmWidth, connector->mmHeight, connector->count_modes);
 
 		if (!connector->count_modes)
 			continue;
 
-		printf("  modes:\n");
-		printf("  name refresh (Hz) hdisp hss hse htot vdisp "
-		       "vss vse vtot flags type clock\n");
+		igt_info("  modes:\n");
+		igt_info("  name refresh (Hz) hdisp hss hse htot vdisp ""vss vse vtot flags type clock\n");
 		for (j = 0; j < connector->count_modes; j++){
-			fprintf(stdout, "[%d]", j );
+			igt_info("[%d]", j);
 			kmstest_dump_mode(&connector->modes[j]);
 		}
 
 		drmModeFreeConnector(connector);
 	}
-	printf("\n");
+	igt_info("\n");
 
 	drmModeFreeResources(mode_resources);
 }
@@ -166,27 +165,22 @@ static void dump_crtcs_fd(int drmfd)
 	int i;
 	drmModeRes *mode_resources = drmModeGetResources(drmfd);
 
-	printf("CRTCs:\n");
-	printf("id\tfb\tpos\tsize\n");
+	igt_info("CRTCs:\n");
+	igt_info("id\tfb\tpos\tsize\n");
 	for (i = 0; i < mode_resources->count_crtcs; i++) {
 		drmModeCrtc *crtc;
 
 		crtc = drmModeGetCrtc(drmfd, mode_resources->crtcs[i]);
 		if (!crtc) {
-			fprintf(stderr, "could not get crtc %i: %s\n",
-				mode_resources->crtcs[i], strerror(errno));
+			igt_warn("could not get crtc %i: %s\n", mode_resources->crtcs[i], strerror(errno));
 			continue;
 		}
-		printf("%d\t%d\t(%d,%d)\t(%dx%d)\n",
-		       crtc->crtc_id,
-		       crtc->buffer_id,
-		       crtc->x, crtc->y,
-		       crtc->width, crtc->height);
+		igt_info("%d\t%d\t(%d,%d)\t(%dx%d)\n", crtc->crtc_id, crtc->buffer_id, crtc->x, crtc->y, crtc->width, crtc->height);
 		kmstest_dump_mode(&crtc->mode);
 
 		drmModeFreeCrtc(crtc);
 	}
-	printf("\n");
+	igt_info("\n");
 
 	drmModeFreeResources(mode_resources);
 }
@@ -203,8 +197,8 @@ static void connector_find_preferred_mode(uint32_t connector_id,
 {
 	struct kmstest_connector_config config;
 
-	if (kmstest_get_connector_config(drm_fd, connector_id, crtc_idx_mask,
-					 &config) < 0) {
+	if (!kmstest_get_connector_config(drm_fd, connector_id, crtc_idx_mask,
+					  &config)) {
 		c->mode_valid = 0;
 		return;
 	}
@@ -247,10 +241,7 @@ paint_color_key(struct igt_fb *fb_info)
 
 static void paint_image(cairo_t *cr, const char *file)
 {
-	int img_x, img_y, img_w, img_h, img_w_o, img_h_o;
-	double img_w_scale, img_h_scale;
-
-	cairo_surface_t *image;
+	int img_x, img_y, img_w, img_h;
 
 	img_y = height * (0.10 );
 	img_h = height * 0.08 * 4;
@@ -258,22 +249,7 @@ static void paint_image(cairo_t *cr, const char *file)
 
 	img_x = (width / 2) - (img_w / 2);
 
-	image = cairo_image_surface_create_from_png(file);
-
-	img_w_o = cairo_image_surface_get_width(image);
-	img_h_o = cairo_image_surface_get_height(image);
-
-	cairo_translate(cr, img_x, img_y);
-
-	img_w_scale = (double)img_w / (double)img_w_o;
-	img_h_scale = (double)img_h / (double)img_h_o;
-	cairo_scale(cr, img_w_scale, img_h_scale);
-
-	cairo_set_source_surface(cr, image, 0, 0);
-	cairo_scale(cr, 1, 1);
-
-	cairo_paint(cr);
-	cairo_surface_destroy(image);
+	igt_paint_image(cr, file, img_x, img_y, img_h, img_w);
 }
 
 static void paint_output_info(struct connector *c, struct igt_fb *fb)
@@ -325,9 +301,11 @@ static void paint_output_info(struct connector *c, struct igt_fb *fb)
 	}
 
 	if (qr_code)
-		paint_image(cr, IGT_DATADIR"/pass.png");
+		paint_image(cr, "pass.png");
 
 	igt_assert(!cairo_status(cr));
+
+	cairo_destroy(cr);
 }
 
 static void sighandler(int signo)
@@ -343,8 +321,8 @@ static void set_single(void)
 
 	sigemptyset(&sa.sa_mask);
 
-	if (sigaction(sigs[0], &sa, NULL) == -1)
-		perror("Could not set signal handler");
+	igt_warn_on_f(sigaction(sigs[0], &sa, NULL) == -1,
+		      "Could not set signal handler");
 }
 
 static void
@@ -379,18 +357,16 @@ set_mode(struct connector *c)
 		height = c->mode.vdisplay;
 
 		fb_id = igt_create_fb(drm_fd, width, height,
-					  igt_bpp_depth_to_drm_format(bpp, depth),
-					  enable_tiling, &fb_info[current_fb]);
+				      igt_bpp_depth_to_drm_format(bpp, depth),
+				      tiling, &fb_info[current_fb]);
 		paint_output_info(c, &fb_info[current_fb]);
 		paint_color_key(&fb_info[current_fb]);
 
-		fprintf(stdout, "CRTC(%u):[%d]",c->crtc, j);
+		igt_info("CRTC(%u):[%d]", c->crtc, j);
 		kmstest_dump_mode(&c->mode);
 		if (drmModeSetCrtc(drm_fd, c->crtc, fb_id, 0, 0,
 				   &c->id, 1, &c->mode)) {
-			fprintf(stderr, "failed to set mode (%dx%d@%dHz): %s\n",
-				width, height, c->mode.vrefresh,
-				strerror(errno));
+			igt_warn("failed to set mode (%dx%d@%dHz): %s\n", width, height, c->mode.vrefresh, strerror(errno));
 			continue;
 		}
 
@@ -421,132 +397,16 @@ set_mode(struct connector *c)
 	drmModeFreeConnector(c->connector);
 }
 
-struct box {
-	int x, y, width, height;
-};
-
-struct stereo_fb_layout {
-	int fb_width, fb_height;
-	struct box left, right;
-};
-
-static void box_init(struct box *box, int x, int y, int bwidth, int bheight)
-{
-	box->x = x;
-	box->y = y;
-	box->width = bwidth;
-	box->height = bheight;
-}
-
-static void stereo_fb_layout_from_mode(struct stereo_fb_layout *layout,
-				       drmModeModeInfo *mode)
-{
-	unsigned int format = mode->flags & DRM_MODE_FLAG_3D_MASK;
-	const int hdisplay = mode->hdisplay, vdisplay = mode->vdisplay;
-	int middle;
-
-	switch (format) {
-	case DRM_MODE_FLAG_3D_TOP_AND_BOTTOM:
-		layout->fb_width = hdisplay;
-		layout->fb_height = vdisplay;
-
-		middle = vdisplay / 2;
-		box_init(&layout->left, 0, 0, hdisplay, middle);
-		box_init(&layout->right,
-			 0, middle, hdisplay, vdisplay - middle);
-		break;
-	case DRM_MODE_FLAG_3D_SIDE_BY_SIDE_HALF:
-		layout->fb_width = hdisplay;
-		layout->fb_height = vdisplay;
-
-		middle = hdisplay / 2;
-		box_init(&layout->left, 0, 0, middle, vdisplay);
-		box_init(&layout->right,
-			 middle, 0, hdisplay - middle, vdisplay);
-		break;
-	case DRM_MODE_FLAG_3D_FRAME_PACKING:
-	{
-		int vactive_space = mode->vtotal - vdisplay;
-
-		layout->fb_width = hdisplay;
-		layout->fb_height = 2 * vdisplay + vactive_space;
-
-		box_init(&layout->left,
-			 0, 0, hdisplay, vdisplay);
-		box_init(&layout->right,
-			 0, vdisplay + vactive_space, hdisplay, vdisplay);
-		break;
-	}
-	default:
-		igt_assert(0);
-	}
-}
-
-static const char *stereo_mode_str(drmModeModeInfo *mode)
-{
-	unsigned int layout = mode->flags & DRM_MODE_FLAG_3D_MASK;
-
-	switch (layout) {
-	case DRM_MODE_FLAG_3D_TOP_AND_BOTTOM:
-		return "TB";
-	case DRM_MODE_FLAG_3D_SIDE_BY_SIDE_HALF:
-		return "SbSH";
-	case DRM_MODE_FLAG_3D_FRAME_PACKING:
-		return "FP";
-	default:
-		igt_assert(0);
-	}
-}
-
-static uint32_t create_stereo_fb(drmModeModeInfo *mode, struct igt_fb *fb)
-{
-	struct stereo_fb_layout layout;
-	cairo_t *cr;
-	uint32_t fb_id;
-
-	stereo_fb_layout_from_mode(&layout, mode);
-	fb_id = igt_create_fb(drm_fd, layout.fb_width, layout.fb_height,
-				  igt_bpp_depth_to_drm_format(bpp, depth),
-				  enable_tiling, fb);
-	cr = igt_get_cairo_ctx(drm_fd, fb);
-
-	igt_paint_image(cr, IGT_DATADIR"/1080p-left.png",
-			    layout.left.x, layout.left.y,
-			    layout.left.width, layout.left.height);
-	igt_paint_image(cr, IGT_DATADIR"/1080p-right.png",
-			    layout.right.x, layout.right.y,
-			    layout.right.width, layout.right.height);
-
-	cairo_destroy(cr);
-
-	{
-		char buffer[64];
-
-		snprintf(buffer, sizeof(buffer), "%dx%d@%dHz-%s.png",
-			 mode->hdisplay,
-			 mode->vdisplay,
-			 mode->vrefresh,
-			 stereo_mode_str(mode));
-
-		igt_write_fb_to_png(drm_fd, fb, buffer);
-	}
-
-	return fb_id;
-}
-
 static void do_set_stereo_mode(struct connector *c)
 {
 	uint32_t fb_id;
-	struct igt_fb fb_info;
 
-	fb_id = create_stereo_fb(&c->mode, &fb_info);
+	fb_id = igt_create_stereo_fb(drm_fd, &c->mode,
+				     igt_bpp_depth_to_drm_format(bpp, depth),
+				     tiling);
 
-	if (drmModeSetCrtc(drm_fd, c->crtc, fb_id, 0, 0,
-			   &c->id, 1, &c->mode)) {
-		fprintf(stderr, "failed to set mode (%dx%d@%dHz): %s\n",
-			width, height, c->mode.vrefresh,
-			strerror(errno));
-	}
+	igt_warn_on_f(drmModeSetCrtc(drm_fd, c->crtc, fb_id, 0, 0, &c->id, 1, &c->mode),
+		      "failed to set mode (%dx%d@%dHz): %s\n", width, height, c->mode.vrefresh, strerror(errno));
 }
 
 static void
@@ -570,7 +430,7 @@ set_stereo_mode(struct connector *c)
 		if (!(c->mode.flags & DRM_MODE_FLAG_3D_MASK))
 			continue;
 
-		fprintf(stdout, "CRTC(%u): [%d]", c->crtc, i);
+		igt_info("CRTC(%u): [%d]", c->crtc, i);
 		kmstest_dump_mode(&c->mode);
 		do_set_stereo_mode(c);
 
@@ -607,8 +467,7 @@ int update_display(void)
 
 	resources = drmModeGetResources(drm_fd);
 	if (!resources) {
-		fprintf(stderr, "drmModeGetResources failed: %s\n",
-			strerror(errno));
+		igt_warn("drmModeGetResources failed: %s\n", strerror(errno));
 		return 0;
 	}
 
@@ -671,27 +530,27 @@ int update_display(void)
 	return 1;
 }
 
-static char optstr[] = "3hiaf:s:d:p:mrto:j:";
+static char optstr[] = "3hiaf:s:d:p:mrto:j:y";
 
-static void __attribute__((noreturn)) usage(char *name)
+static void __attribute__((noreturn)) usage(char *name, char opt)
 {
-	fprintf(stderr, "usage: %s [-hiasdpmtf]\n", name);
-	fprintf(stderr, "\t-i\tdump info\n");
-	fprintf(stderr, "\t-a\ttest all modes\n");
-	fprintf(stderr, "\t-s\t<duration>\tsleep between each mode test\n");
-	fprintf(stderr, "\t-d\t<depth>\tbit depth of scanout buffer\n");
-	fprintf(stderr, "\t-p\t<planew,h>,<crtcx,y>,<crtcw,h> test overlay plane\n");
-	fprintf(stderr, "\t-m\ttest the preferred mode\n");
-	fprintf(stderr, "\t-3\ttest all 3D modes\n");
-	fprintf(stderr, "\t-t\tuse a tiled framebuffer\n");
-	fprintf(stderr, "\t-j\tdo dpms off, optional arg to select dpms leve (1-3)\n");
-	fprintf(stderr, "\t-r\tprint a QR code on the screen whose content is \"pass\" for the automatic test\n");
-	fprintf(stderr, "\t-o\t<id of the display>,<number of the mode>\tonly test specified mode on the specified display\n");
-	fprintf(stderr, "\t-f\t<clock MHz>,<hdisp>,<hsync-start>,<hsync-end>,<htotal>,\n");
-	fprintf(stderr, "\t\t<vdisp>,<vsync-start>,<vsync-end>,<vtotal>\n");
-	fprintf(stderr, "\t\ttest force mode\n");
-	fprintf(stderr, "\tDefault is to test all modes.\n");
-	exit(0);
+	igt_info("usage: %s [-hiasdpmtf]\n", name);
+	igt_info("\t-i\tdump info\n");
+	igt_info("\t-a\ttest all modes\n");
+	igt_info("\t-s\t<duration>\tsleep between each mode test\n");
+	igt_info("\t-d\t<depth>\tbit depth of scanout buffer\n");
+	igt_info("\t-p\t<planew,h>,<crtcx,y>,<crtcw,h> test overlay plane\n");
+	igt_info("\t-m\ttest the preferred mode\n");
+	igt_info("\t-3\ttest all 3D modes\n");
+	igt_info("\t-t\tuse a tiled framebuffer\n");
+	igt_info("\t-j\tdo dpms off, optional arg to select dpms leve (1-3)\n");
+	igt_info("\t-r\tprint a QR code on the screen whose content is \"pass\" for the automatic test\n");
+	igt_info("\t-o\t<id of the display>,<number of the mode>\tonly test specified mode on the specified display\n");
+	igt_info("\t-f\t<clock MHz>,<hdisp>,<hsync-start>,<hsync-end>,<htotal>,\n");
+	igt_info("\t\t<vdisp>,<vsync-start>,<vsync-end>,<vtotal>\n");
+	igt_info("\t\ttest force mode\n");
+	igt_info("\tDefault is to test all modes.\n");
+	exit((opt != 'h') ? -1 : 0);
 }
 
 #define dump_resource(res) if (res) dump_##res()
@@ -732,7 +591,7 @@ static void enter_exec_path( char **argv )
 		*(pos+1) = '\0';
 
 	ret = chdir(exec_path);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 	free(exec_path);
 }
 
@@ -746,6 +605,11 @@ static void set_termio_mode(void)
 {
 	struct termios tio;
 
+	/* don't attempt to set terminal attributes if not in the foreground
+	 * process group */
+	if (getpgrp() != tcgetpgrp(STDOUT_FILENO))
+		return;
+
 	tio_fd = dup(STDIN_FILENO);
 	tcgetattr(tio_fd, &saved_tio);
 	igt_install_exit_handler(restore_termio_mode);
@@ -762,12 +626,21 @@ int main(int argc, char **argv)
 	GMainLoop *mainloop;
 	float force_clock;
 	bool opt_dump_info = false;
+	struct option long_opts[] = {
+		{"list-subtests", 0, 0, SUBTEST_OPTS},
+		{"run-subtest", 1, 0, SUBTEST_OPTS},
+		{"help-description", 0, 0, HELP_DESCRIPTION},
+		{"help", 0, 0, 'h'},
+		{"yb", 0, 0, Yb_OPT},
+		{"yf", 0, 0, Yf_OPT},
+		{ 0, 0, 0, 0 }
+	};
 
 	igt_skip_on_simulation();
 
 	enter_exec_path( argv );
 
-	while ((c = getopt(argc, argv, optstr)) != -1) {
+	while ((c = getopt_long(argc, argv, optstr, long_opts, NULL)) != -1) {
 		switch (c) {
 		case '3':
 			test_stereo_modes = 1;
@@ -783,7 +656,7 @@ int main(int argc, char **argv)
 			if(sscanf(optarg,"%f,%hu,%hu,%hu,%hu,%hu,%hu,%hu,%hu",
 				&force_clock,&force_timing.hdisplay, &force_timing.hsync_start,&force_timing.hsync_end,&force_timing.htotal,
 				&force_timing.vdisplay, &force_timing.vsync_start, &force_timing.vsync_end, &force_timing.vtotal)!= 9)
-				usage(argv[0]);
+				usage(argv[0], c);
 			force_timing.clock = force_clock*1000;
 
 			break;
@@ -797,20 +670,27 @@ int main(int argc, char **argv)
 			break;
 		case 'd':
 			depth = atoi(optarg);
-			fprintf(stderr, "using depth %d\n", depth);
+			igt_info("using depth %d\n", depth);
 			break;
 		case 'p':
 			if (sscanf(optarg, "%d,%d,%d,%d,%d,%d", &plane_width,
 				   &plane_height, &crtc_x, &crtc_y,
 				   &crtc_w, &crtc_h) != 6)
-				usage(argv[0]);
+				usage(argv[0], c);
 			test_plane = 1;
 			break;
 		case 'm':
 			test_preferred_mode = 1;
 			break;
 		case 't':
-			enable_tiling = 1;
+			tiling = LOCAL_I915_FORMAT_MOD_X_TILED;
+			break;
+		case 'y':
+		case Yb_OPT:
+			tiling = LOCAL_I915_FORMAT_MOD_Y_TILED;
+			break;
+		case Yf_OPT:
+			tiling = LOCAL_I915_FORMAT_MOD_Yf_TILED;
 			break;
 		case 'r':
 			qr_code = 1;
@@ -818,10 +698,18 @@ int main(int argc, char **argv)
 		case 'o':
 			sscanf(optarg, "%d,%d", &specified_disp_id, &specified_mode_num);
 			break;
+		case SUBTEST_OPTS:
+			/* invalid subtest options */
+			exit(IGT_EXIT_INVALID);
+			break;
+		case HELP_DESCRIPTION:
+			igt_info("Tests display functionality.");
+			exit(0);
+			break;
 		default:
 			/* fall through */
 		case 'h':
-			usage(argv[0]);
+			usage(argv[0], c);
 			break;
 		}
 	}
@@ -843,7 +731,7 @@ int main(int argc, char **argv)
 
 	if (test_stereo_modes &&
 	    drmSetClientCap(drm_fd, DRM_CLIENT_CAP_STEREO_3D, 1) < 0) {
-		fprintf(stderr, "DRM_CLIENT_CAP_STEREO_3D failed\n");
+		igt_warn("DRM_CLIENT_CAP_STEREO_3D failed\n");
 		goto out_close;
 	}
 
@@ -852,30 +740,30 @@ int main(int argc, char **argv)
 		goto out_close;
 	}
 
-	igt_set_vt_graphics_mode();
+	kmstest_set_vt_graphics_mode();
 
 	mainloop = g_main_loop_new(NULL, FALSE);
 	if (!mainloop) {
-		fprintf(stderr, "failed to create glib mainloop\n");
+		igt_warn("failed to create glib mainloop\n");
 		ret = -1;
 		goto out_close;
 	}
 
 	if (!testdisplay_setup_hotplug()) {
-		fprintf(stderr, "failed to initialize hotplug support\n");
+		igt_warn("failed to initialize hotplug support\n");
 		goto out_mainloop;
 	}
 
 	stdinchannel = g_io_channel_unix_new(0);
 	if (!stdinchannel) {
-		fprintf(stderr, "failed to create stdin GIO channel\n");
+		igt_warn("failed to create stdin GIO channel\n");
 		goto out_hotplug;
 	}
 
 	ret = g_io_add_watch(stdinchannel, G_IO_IN | G_IO_ERR, input_event,
 			     NULL);
 	if (ret < 0) {
-		fprintf(stderr, "failed to add watch on stdin GIO channel\n");
+		igt_warn("failed to add watch on stdin GIO channel\n");
 		goto out_stdio;
 	}
 
@@ -900,5 +788,7 @@ out_mainloop:
 out_close:
 	close(drm_fd);
 
-	return ret;
+	igt_assert_eq(ret, 0);
+
+	igt_exit();
 }
