@@ -29,12 +29,15 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <err.h>
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
+#include "intel_chipset.h"
 
 int main(int argc, char** argv)
 {
 	uint32_t reg, value;
-	volatile uint32_t *ptr;
+
+	fprintf(stderr, "WARNING: Use of %s has been deprecated and replaced by"
+		" intel_reg.\n", argv[0]);
 
 	if (argc < 3) {
 		printf("Usage: %s addr value\n", argv[0]);
@@ -46,11 +49,10 @@ int main(int argc, char** argv)
 	intel_register_access_init(intel_get_pci_device(), 0);
 	sscanf(argv[1], "0x%x", &reg);
 	sscanf(argv[2], "0x%x", &value);
-	ptr = (volatile uint32_t *)((volatile char *)mmio + reg);
 
-	printf("Value before: 0x%X\n", *ptr);
-	*ptr = value;
-	printf("Value after: 0x%X\n", *ptr);
+	printf("Value before: 0x%X\n", INREG(reg));
+	OUTREG(reg, value);
+	printf("Value after: 0x%X\n", INREG(reg));
 
 	intel_register_access_fini();
 	return 0;
